# Axis Group Jogging URCap Example
## Description
The Axis Group Jogging URCap provides a working example for jogging the axis in a given axis_group-axis pair via control presented in the toolbar.
## Build
* The URCap SDK must be installed and available in the build environment
1.
   ```bash
   cd axis-group-jogging
   ```
1.
   ```bash
   ./build.sh
   ```
   Note: By default the build script will attempt to build the URCap then deploy to a local simulator (if available).
## Usage
* Note: The Axis Group Jogging URCap does not manage the lifecycle of any world-model or axis-group entities.
1. On the robot, execute a program or script that creates an Axis Group and with 2 Axes
   * The program/script should finish without removing the Axis Group and the Axis.
   * Example script:
     ```ruby
     AXIS_TYPE = 0 # 0:rotary, 1:linear
     AXIS_VELOCITY_LIMIT = d2r(360) # rad/s
     AXIS_ACCELERATION_LIMIT = d2r(3600) # rad/s^2
     AXIS_ENCODER_RESOLUTION = 3600 # counts/rev
     AXIS_FEED_CONSTANT = d2r(360) # rad/rev
     AXIS_GEAR_RATIO = 1.0
     AXIS_ZERO_OFFSET = 0

     reset_world_model()
     axis_group_add("group1", p[0, 0, 0, 0, 0, 0], "world")
     axis_group_add_axis("group1", "axis1", "", p[0, 0, 0, 0, 0, 0], AXIS_TYPE, AXIS_VELOCITY_LIMIT, AXIS_ACCELERATION_LIMIT)
     axis_group_add_axis("group1", "axis2", "", p[0, 0, 0, 0, 0, 0], AXIS_TYPE, AXIS_VELOCITY_LIMIT, AXIS_ACCELERATION_LIMIT)
     ```
1. After the program has finished executing, open the Axis Jog panel of the toolbar

   ![open toolbar](./docs/images/open_toolbar.png)
1. Select the desired Axis to jog, and enter the preferred parameters specific to the Axis

   ![toolbar parameters](./docs/images/toolbar_parameters.png)
1. Jog the given Axis by holding down either the `<-` or `->` button
   * The grey indicator icon will change to green and the `Sent Command` field will update to indicate the command that has been sent to the robot via the Primary Interface
   
   ![activate jogging](./docs/images/activate_jogging.png)
1. Halt jogging by releasing the held-down button
   * The green indicator icon will change to grey and the `Sent Command` field will update to indicate the command that has been sent to the robot via the Primary Interface
   
   ![deactivate jogging](./docs/images/deactivate_jogging.png)
* Notes:
  1. While the toolbar is open, the `actual_external_q` and `target_external_q` fields will reflect the RTDE registers of the same names
  1. If operating without external axis hardware (e.g. Ethercat URCap and servo), only the `target_external_q` value will change
