package com.ur.urcap.externalaxis.axisgroupjogging.URCommunicator;

import java.io.BufferedReader;
import java.io.Closeable;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;

public class ScriptSender {
    // Port for primary client
    public static final int PRIMARY_PORT = 30001;
    public static final int RETURN_PORT = 5500;
    private volatile Socket socket;
    private DataOutputStream outputStream;


    public synchronized boolean connect(String host, int port, int timeout) {
        if (this.socket != null) {
            return this.socket.isConnected();
        }
        try {
            this.socket = new Socket();
            this.socket.setKeepAlive(true);
            this.socket.setSoTimeout(timeout);
            this.socket.connect(new InetSocketAddress(host, port));
            this.outputStream = new DataOutputStream(socket.getOutputStream());
            return true;
        } catch (IOException e) {
            System.out.println(String.format("Could not connect to %s:%s", host, port));
            System.out.println(e);
            disconnect();
            return false;
        }
    }

    public boolean isConnected() {
        return this.socket != null && this.socket.isConnected();
    }

    public synchronized boolean disconnect() {
        if (this.socket == null) {
            return true;
        }
        closeSilently(this.outputStream);
        this.outputStream = null;
        closeSilently(this.socket);
        this.socket = null;
        return true;
    }

    /**
     * Method used to send a ScriptCommand as a primary program to the connected
     * Interface.
     * If called while a program is already running, the existing program will halt.
     * 
     * @param command the ScriptCommand object to send.
     */
    public void sendScriptCommand(ScriptCommand command) {
        try {
            if (isConnected()) {
                // Send command
                this.outputStream.write(command.toString().getBytes("US-ASCII"));
                this.outputStream.flush();
            } else {
                System.out.println("Cannot send script: Not connected");
            }
        } catch (IOException e) {
            System.out.println(e);
        }
    }

    /**
     * Checks if the robot is running prior to sending the URscript to the connected
     * interface of the UR-controller.
     *
     * @return success
     */
    public boolean executeURScript(ScriptCommand command) {
        DashboardServerCommunicator dashboard = new DashboardServerCommunicator();
        try {
            String robotmode = dashboard.robotmode();

            if (robotmode.matches("Robotmode: RUNNING")) {
                sendScriptCommand(command);
                return true;
            } else {
                System.out.println("Robot is not running, failed to send script");
                dashboard.popup("Robot is not running");
                return false;
            }

        } catch (IOException e) {
            e.printStackTrace();
        }

        return false;
    }

    public String executeURScriptWithStringReturn(ScriptCommand command) {
        String result = "";
        if (executeURScript(command)) {
            try {
                ServerSocket server = new ServerSocket(RETURN_PORT);
                server.setSoTimeout(200);

                Socket returnSocket;
                try {
                     returnSocket = server.accept();
                } catch (SocketTimeoutException e) {
                    server.close();
                    throw e;
                }
                BufferedReader readerFromURScript = new BufferedReader(new InputStreamReader(returnSocket.getInputStream()));
                result = readerFromURScript.readLine();
                // Housekeeping
                readerFromURScript.close();
                returnSocket.close();
                server.close();
            } catch (IOException e) {
                System.out.println(e);
            }
        }
        return result;
    }

    private static void closeSilently(Closeable closeable) {
        try {
            if (closeable == null) {
                return;
            }
            closeable.close();
        } catch (IOException e) {
        }
    }
}
