/*
MIT License

Copyright (c) 2019 Oleksandr Shaposhnikov

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

https://github.com/o5h/skynet
*/
package com.ur.urcap.externalaxis.axisgroupjogging.rtde;

import java.io.*;

public class RTDEInputStream implements Closeable {

    private final DataInputStream is;

    public RTDEInputStream(InputStream is) {
        this.is = new DataInputStream(is);
    }

    public int available() throws IOException {
        return is.available();
    }

    public boolean readBool() throws IOException {
        return is.readUnsignedByte() != 0;
    }

    public byte readUInt8() throws IOException {
        return (byte)is.readUnsignedByte();
    }

    public short readUInt16() throws IOException {
        return (short)is.readUnsignedShort();
    }

    public int readInt32() throws IOException {
        return is.readInt();
    }

    public int readUInt32() throws IOException {
        return is.readInt();
    }

    public long readUInt64() throws IOException {
        return is.readLong();
    }

    public double readDouble() throws IOException {
        return is.readDouble();
    }

    public void readFully(byte[] data) throws IOException {
        this.is.readFully(data);
    }

    public double[] readVector6Double() throws IOException {
        double[] vec = new double[6];
        vec[0] = readDouble();
        vec[1] = readDouble();
        vec[2] = readDouble();
        vec[3] = readDouble();
        vec[4] = readDouble();
        vec[5] = readDouble();
        return vec;
    }

    public double[] readVector3Double() throws IOException {
        double[] vec = new double[3];
        vec[0] = readDouble();
        vec[1] = readDouble();
        vec[2] = readDouble();
        return vec;
    }

    public int[] readVector6Int32() throws IOException {
        int[] vec = new int[6];
        vec[0] = readInt32();
        vec[1] = readInt32();
        vec[2] = readInt32();
        vec[3] = readInt32();
        vec[4] = readInt32();
        vec[5] = readInt32();
        return vec;
    }

    public long[] readVector6UInt32() throws IOException {
        long[] vec = new long[6];
        vec[0] = readUInt32();
        vec[1] = readUInt32();
        vec[2] = readUInt32();
        vec[3] = readUInt32();
        vec[4] = readUInt32();
        vec[5] = readUInt32();
        return vec;
    }

    @Override
    public void close() throws IOException {
        is.close();
    }

}