/*
MIT License

Copyright (c) 2019 Oleksandr Shaposhnikov

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

https://github.com/o5h/skynet
*/
package com.ur.urcap.externalaxis.axisgroupjogging.rtde;


import java.io.Closeable;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;

public class RTDEOutputStream implements Closeable {
    private final DataOutputStream os;

    public RTDEOutputStream(OutputStream os) {
        this.os = new DataOutputStream(os);
    }

    public void writeUInt8(int value) throws IOException {
        this.os.writeByte(value);
    }

    public void writeUInt16(int value) throws IOException {
        this.os.writeShort(value);
    }

    public void writeUInt32(int value) throws IOException {
        this.os.writeInt(value);
    }

    public void writeUInt64(long value) throws IOException {
        this.os.writeLong(value);
    }

    public void writeInt32(int value) throws IOException {
        this.os.writeInt(value);
    }

    public void flush() throws IOException {
        this.os.flush();
    }

    public void write(byte[] data) throws IOException {
        os.write(data);
    }

    public void writeDouble(double v) throws IOException {
        this.os.writeDouble(v);
    }

    public void writeVector3d(double[] v) throws IOException {
        this.os.writeDouble(v[0]);
        this.os.writeDouble(v[1]);
        this.os.writeDouble(v[2]);
    }

    public void writeVector6d(double[] v) throws IOException {
        this.os.writeDouble(v[0]);
        this.os.writeDouble(v[1]);
        this.os.writeDouble(v[2]);
        this.os.writeDouble(v[3]);
        this.os.writeDouble(v[4]);
        this.os.writeDouble(v[5]);
    }

    public void writeVector6i(int[] v) throws IOException {
        this.os.writeInt(v[0]);
        this.os.writeInt(v[1]);
        this.os.writeInt(v[2]);
        this.os.writeInt(v[3]);
        this.os.writeInt(v[4]);
        this.os.writeInt(v[5]);
    }

    public void writeVector6u(int[] v) throws IOException {
        this.os.writeInt(v[0]);
        this.os.writeInt(v[1]);
        this.os.writeInt(v[2]);
        this.os.writeInt(v[3]);
        this.os.writeInt(v[4]);
        this.os.writeInt(v[5]);
    }

    @Override
    public void close() throws IOException {
        this.os.close();
    }
}