package com.ur.urcap.externalaxis.axisgroupjogging.toolbar;

import com.ur.urcap.api.contribution.toolbar.ToolbarAPIProvider;
import com.ur.urcap.api.contribution.toolbar.ToolbarContext;
import com.ur.urcap.api.contribution.toolbar.swing.SwingToolbarContribution;
import com.ur.urcap.api.domain.userinteraction.inputvalidation.InputValidationFactory;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardInputCallback;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardInputFactory;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardNumberInput;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardTextInput;
import com.ur.urcap.externalaxis.axisgroupjogging.impl.Style;
import com.ur.urcap.externalaxis.axisgroupjogging.impl.V5Style;
import com.ur.urcap.externalaxis.axisgroupjogging.rtde.RTDEClient;
import com.ur.urcap.externalaxis.axisgroupjogging.rtde.RTDEOutputHandler;
import com.ur.urcap.externalaxis.axisgroupjogging.rtde.RTDEOutputParam;
import com.ur.urcap.externalaxis.axisgroupjogging.URCommunicator.ScriptCommand;
import com.ur.urcap.externalaxis.axisgroupjogging.URCommunicator.ScriptSender;

import javax.swing.*;
import javax.swing.event.PopupMenuListener;
import javax.swing.event.PopupMenuEvent;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.stream.Collectors;

class AxisGroupJoggingToolbarContribution implements SwingToolbarContribution {

	private final ToolbarAPIProvider apiProvider;
	private final InputValidationFactory validatorFactory;
	private final KeyboardInputFactory keyboardInputFactory;

	private static final int MAXIMUM_NUMBER_AXES = 6;
	private int selectedGroupSize;
	private int selectedAxisIndex;

	// GUI
	private static final int MAXIMUM_FRACTION_DIGITS = 3;
	private final Style style;
	private ImageIcon axisGroupJogNegativeIcon;
	private ImageIcon axisGroupJogPositiveIcon;
	private ImageIcon axisGroupJogActiveIcon;
	private ImageIcon axisGroupJogInactiveIcon;
	private JButton axisGroupJogNegativeButton = new JButton();
	private JButton axisGroupJogPositiveButton = new JButton();
	private JLabel axisGroupJogStatus = new JLabel();
	private JComboBox groupAxisPairComboBox = new JComboBox();
	private JTextField axisGroupMaxVelocity = new JTextField("0.0");
	private JTextField axisGroupAccelerationFactor = new JTextField("0.0");
	private JLabel axisGroupActualPositions = new JLabel("[0.0, 0.0, 0.0, 0.0, 0.0, 0.0");
	private JLabel axisGroupTargetPositions = new JLabel("[0.0, 0.0, 0.0, 0.0, 0.0, 0.0");
	private JTextArea axisGroupActiveCommand = new JTextArea("");
	private DecimalFormat df = new DecimalFormat("#");

	// ScriptCommunicator
	private final ScriptSender sender;

	// RTDE
	private final RTDEClient client;

	AxisGroupJoggingToolbarContribution(ToolbarContext context) {
		apiProvider = context.getAPIProvider();
		style = new V5Style();
		keyboardInputFactory = apiProvider.getUserInterfaceAPI().getUserInteraction().getKeyboardInputFactory();
		validatorFactory = apiProvider.getUserInterfaceAPI().getUserInteraction().getInputValidationFactory();

		sender = new ScriptSender();

		client = new RTDEClient(new RTDEOutputHandler() {
			@Override
			public void onProtocolVersion(int protocolVersion) {
				if (protocolVersion != 2) {
					throw new RuntimeException("Protocol v2 expected");
				}
			}

			@Override
			public void onData(RTDEOutputParam output, Object value) {
				switch (output) {
					case actual_external_q: {
						double[] data = (double[]) value;
						String stringData = Arrays.stream(data)
							.mapToObj(df::format)
							.collect(Collectors.joining(", "));
						axisGroupActualPositions.setText(stringData);
						break;
					}
					case target_external_q: {
						double[] data = (double[]) value;
						String stringData = Arrays.stream(data)
							.mapToObj(df::format)
							.collect(Collectors.joining(", "));
						axisGroupTargetPositions.setText(stringData);
						break;
					}
					default:
						break;
				}
			}
		});

		df.setMaximumFractionDigits(MAXIMUM_FRACTION_DIGITS);
	}

	@Override
	public void openView() {
		sender.connect("127.0.0.1", ScriptSender.PRIMARY_PORT, 200);
		if (client.connect("127.0.0.1", RTDEClient.PORT, 200)) {
			try {
				client.requestProtocolVersion(2);
				client.setupOutputsV2(50,
					RTDEOutputParam.actual_external_q,
					RTDEOutputParam.target_external_q);
				client.startOutput();
			} catch (Exception e) {
				System.out.println("RTDE client setup failed");
				System.out.println(e);
			}
		}
	}

	@Override
	public void closeView() {
		sender.disconnect();
		client.disconnect();
	}

	public void buildUI(JPanel jPanel) {
		axisGroupJogNegativeIcon = new ImageIcon(getClass().getResource("/icons/left_arrow3.png"));
		axisGroupJogPositiveIcon = new ImageIcon(getClass().getResource("/icons/right_arrow3.png"));
		axisGroupJogActiveIcon = new ImageIcon(getClass().getResource("/icons/G5_led_green.png"));
		axisGroupJogInactiveIcon = new ImageIcon(getClass().getResource("/icons/G5_led_off.png"));

		JLabel groupAxisPairLabel = new JLabel("Group::Axis Pair:");
		setupLabel(groupAxisPairLabel, style.getLabelSize());
		setupComboBox(groupAxisPairComboBox, new PopupMenuListener() {
			@Override
			public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
				selectedGroupSize = 0;
				selectedAxisIndex = 0;
				groupAxisPairComboBox.removeAllItems();
				String[] pairs = getGroupAxisPairs();
				for (String node : pairs) {
					groupAxisPairComboBox.addItem(node);
				}
			}

			@Override
			public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
				selectedGroupSize = 0;
				selectedAxisIndex = 0;
				String[] selected = groupAxisPairComboBox.getSelectedItem().toString().split("::");
				for (int ii = 0; ii < groupAxisPairComboBox.getItemCount(); ii++) {
					String[] item = groupAxisPairComboBox.getItemAt(ii).toString().split("::");
					if (item[0].equals(selected[0])) {
						selectedGroupSize++;
						if (ii < groupAxisPairComboBox.getSelectedIndex()) {
							selectedAxisIndex++;
						}
					}
				}
			}

			@Override
			public void popupMenuCanceled(PopupMenuEvent e) {
			}
		});

		JLabel axisGroupMaxVelocityLabel = new JLabel("Max Velocity:");
		setupLabel(axisGroupMaxVelocityLabel, style.getLabelSize());
		setupTextInputField(axisGroupMaxVelocity, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardNumberInput<Double> keyboard = getKeyboardForAxisGroupMaxVelocity();
				keyboard.show(axisGroupMaxVelocity, getCallbackForAxisGroupMaxVelocity());
			}
		});

		JLabel axisGroupAccelerationFactorLabel = new JLabel("Acceleration Factor:");
		setupLabel(axisGroupAccelerationFactorLabel, style.getLabelSize());
		setupTextInputField(axisGroupAccelerationFactor, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardNumberInput<Double> keyboard = getKeyboardForAxisGroupAccelerationFactor();
				keyboard.show(axisGroupAccelerationFactor, getCallbackForAxisGroupAccelerationFactor());
			}
		});

		JLabel axisGroupJogLabel = new JLabel("Jog:");
		setupLabel(axisGroupJogLabel, style.getSmallLabelSize());
		setupAxisGroupJogControl(
			axisGroupJogNegativeButton,
			axisGroupJogNegativeIcon,
			new MouseAdapter() {
				@Override
				public void mousePressed(MouseEvent e) {
					startJogAxisGroupNegative();
				}

				@Override
				public void mouseReleased(MouseEvent e) {
					stopAxisGroup();
				}
			});

		setupAxisGroupJogControl(
			axisGroupJogPositiveButton,
			axisGroupJogPositiveIcon,
			new MouseAdapter() {
				@Override
				public void mousePressed(MouseEvent e) {
					startJogAxisGroupPositive();
				}

				@Override
				public void mouseReleased(MouseEvent e) {
					stopAxisGroup();
				}
			});

		setupAxisGroupJogStatus(axisGroupJogStatus, axisGroupJogInactiveIcon);

		JLabel axisGroupActualPositionsLabel = new JLabel("actual_external_q:");
		setupDisplayField(axisGroupActualPositions);

		JLabel axisGroupTargetPositionsLabel = new JLabel("target_external_q:");
		setupDisplayField(axisGroupTargetPositions);

		JLabel axisGroupActiveCommandLabel = new JLabel("Sent Command:");
		setupDebugField(axisGroupActiveCommand);

		jPanel.setLayout(new BoxLayout(jPanel, BoxLayout.Y_AXIS));

		JPanel axisGroupPanel = new JPanel();
		GroupLayout layout = new GroupLayout(axisGroupPanel);
		axisGroupPanel.setLayout(layout);
		layout.setAutoCreateGaps(true);
		layout.setAutoCreateContainerGaps(true);

		layout.setHorizontalGroup(
			layout.createSequentialGroup()
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
					.addComponent(groupAxisPairLabel)
					.addComponent(axisGroupMaxVelocityLabel)
					.addComponent(axisGroupAccelerationFactorLabel)
					.addGroup(layout.createSequentialGroup()
						.addComponent(axisGroupJogLabel)
						.addComponent(axisGroupJogStatus)
					)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
					.addComponent(groupAxisPairComboBox)
					.addComponent(axisGroupMaxVelocity)
					.addComponent(axisGroupAccelerationFactor)
					.addGroup(layout.createSequentialGroup()
						.addComponent(axisGroupJogNegativeButton)
						.addComponent(axisGroupJogPositiveButton)
					)
				)
		);

		layout.setVerticalGroup(
			layout.createSequentialGroup()
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
					.addComponent(groupAxisPairLabel)
					.addComponent(groupAxisPairComboBox)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
					.addComponent(axisGroupMaxVelocityLabel)
					.addComponent(axisGroupMaxVelocity)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
					.addComponent(axisGroupAccelerationFactorLabel)
					.addComponent(axisGroupAccelerationFactor)
				)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
					.addComponent(axisGroupJogLabel)
					.addComponent(axisGroupJogStatus)
					.addComponent(axisGroupJogNegativeButton)
					.addComponent(axisGroupJogPositiveButton)
				)
		);

		JPanel rtdePanel = new JPanel();
		GroupLayout rtdeLayot = new GroupLayout(rtdePanel);
		rtdePanel.setLayout(rtdeLayot);
		rtdePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "RTDE Data"));
		rtdeLayot.setAutoCreateGaps(true);
		rtdeLayot.setAutoCreateContainerGaps(true);

		rtdeLayot.setHorizontalGroup(
			rtdeLayot.createSequentialGroup()
				.addGroup(rtdeLayot.createParallelGroup(GroupLayout.Alignment.LEADING)
					.addComponent(axisGroupActualPositionsLabel)
					.addComponent(axisGroupTargetPositionsLabel)
				)
				.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, style.getExtraLargeVerticalSpacing(), Short.MAX_VALUE)
				.addGroup(rtdeLayot.createParallelGroup(GroupLayout.Alignment.LEADING)
					.addComponent(axisGroupActualPositions)
					.addComponent(axisGroupTargetPositions)
				)
		);

		rtdeLayot.setVerticalGroup(
			rtdeLayot.createSequentialGroup()
				.addGroup(rtdeLayot.createParallelGroup(GroupLayout.Alignment.BASELINE)
					.addComponent(axisGroupActualPositionsLabel)
					.addComponent(axisGroupActualPositions)
				)
				.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, style.getVerticalSpacing(), Short.MAX_VALUE)
				.addGroup(rtdeLayot.createParallelGroup(GroupLayout.Alignment.BASELINE)
					.addComponent(axisGroupTargetPositionsLabel)
					.addComponent(axisGroupTargetPositions)
				)
		);

		JPanel commandPanel = new JPanel();
		GroupLayout commandLayout = new GroupLayout(commandPanel);
		commandPanel.setLayout(commandLayout);
		rtdeLayot.setAutoCreateGaps(true);
		rtdeLayot.setAutoCreateContainerGaps(true);

		commandLayout.setHorizontalGroup(
			commandLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addComponent(axisGroupActiveCommandLabel)
				.addComponent(axisGroupActiveCommand)
		);

		commandLayout.setVerticalGroup(
			commandLayout.createSequentialGroup()
				.addComponent(axisGroupActiveCommandLabel)
				.addComponent(axisGroupActiveCommand)
		);

		axisGroupPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		axisGroupPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		rtdePanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		rtdePanel.setAlignmentY(Component.TOP_ALIGNMENT);
		commandPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		commandPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		jPanel.add(axisGroupPanel);
		jPanel.add(rtdePanel);
		jPanel.add(commandPanel);
	}

	private void setupTextInputField(final JTextField inputField, MouseAdapter mouseAdapter) {
		inputField.setFocusable(false);
		inputField.setPreferredSize(style.getInputfieldSize());
		inputField.setMaximumSize(inputField.getPreferredSize());
		inputField.addMouseListener(mouseAdapter);
	}

	private void setupLabel(final JLabel label, final Dimension dim) {
		label.setFocusable(false);
		label.setPreferredSize(dim);
		label.setMaximumSize(label.getPreferredSize());
	}

	private void setupComboBox(final JComboBox box, PopupMenuListener menuListener) {
		box.setFocusable(false);
		box.setPreferredSize(style.getInputfieldSize());
		box.setMaximumSize(box.getPreferredSize());
		box.addPopupMenuListener(menuListener);
	}

	private KeyboardNumberInput<Double> getKeyboardForAxisGroupMaxVelocity() {
		KeyboardNumberInput<Double> keyboardInput = keyboardInputFactory.createPositiveDoubleKeypadInput();
		keyboardInput.setInitialValue(Double.parseDouble(axisGroupMaxVelocity.getText()));
		return keyboardInput;
	}

	private KeyboardInputCallback<Double> getCallbackForAxisGroupMaxVelocity() {
		return new KeyboardInputCallback<Double>() {
			@Override
			public void onOk(Double value) {
				String stringValue = df.format((double) value);
				axisGroupMaxVelocity.setText(stringValue);
			}
		};
	}

	private KeyboardNumberInput<Double> getKeyboardForAxisGroupAccelerationFactor() {
		KeyboardNumberInput<Double> keyboardInput = keyboardInputFactory.createPositiveDoubleKeypadInput();
		keyboardInput.setInitialValue(Double.parseDouble(axisGroupAccelerationFactor.getText()));
		keyboardInput.setErrorValidator(validatorFactory.createDoubleRangeValidator(0.0, 1.0));
		return keyboardInput;
	}

	private KeyboardInputCallback<Double> getCallbackForAxisGroupAccelerationFactor() {
		return new KeyboardInputCallback<Double>() {
			@Override
			public void onOk(Double value) {
				String stringValue = df.format((double) value);
				axisGroupAccelerationFactor.setText(stringValue);
			}
		};
	}

	private void setupAxisGroupJogControl(
			final JButton jogButton,
			final ImageIcon jogIcon,
			MouseAdapter jogMouseAdapter) {
		jogButton.setIcon(jogIcon);
		jogButton.setFocusable(false);
		jogButton.setPreferredSize(style.getButtonSize());
		jogButton.setMaximumSize(jogButton.getPreferredSize());
		jogButton.addMouseListener(jogMouseAdapter);
	}

	private void setupAxisGroupJogStatus(
			final JLabel jogStatusLabel,
			final ImageIcon jogInactiveIcon) {
		jogStatusLabel.setIcon(jogInactiveIcon);
		jogStatusLabel.setFocusable(false);
		jogStatusLabel.setPreferredSize(style.getSmallButtonSize());
		jogStatusLabel.setMaximumSize(jogStatusLabel.getPreferredSize());
	}

	private void setupDisplayField(final JLabel displayLabel) {
		displayLabel.setFocusable(false);
		displayLabel.setPreferredSize(style.getDisplayfieldSize());
		displayLabel.setMaximumSize(displayLabel.getPreferredSize());
	}

	private void setupDebugField(final JTextArea debugArea) {
		debugArea.setFocusable(false);
		debugArea.setFont(new Font(Font.MONOSPACED, Font.PLAIN, style.getDebugFontSize()));
	}

	private void startJogAxisGroupNegative() {
		double[] velocities = new double[selectedGroupSize];
		try {
			velocities[selectedAxisIndex] = -Double.valueOf(axisGroupMaxVelocity.getText());
		} catch (Exception e) {
			System.err.println("Invalid number format: " + e.getMessage());
		}
		ScriptCommand cmd = new ScriptCommand("urcap_external_axis_jog_negative");
		cmd.appendLine(String.format(
			"axis_group_speedj(\"%s\", [%s], %s, 100)",
			groupAxisPairComboBox.getSelectedItem().toString().split("::")[0],
			Arrays.stream(velocities).mapToObj(df::format).collect(Collectors.joining(", ")),
			axisGroupAccelerationFactor.getText()));
		if (sender.executeURScript(cmd)) {
			axisGroupJogStatus.setIcon(axisGroupJogActiveIcon);
			axisGroupActiveCommand.setText(cmd.toString());
		}

	}

	private void startJogAxisGroupPositive() {
		double[] velocities = new double[selectedGroupSize];
		try {
			velocities[selectedAxisIndex] = Double.valueOf(axisGroupMaxVelocity.getText());
		} catch (Exception e) {
			System.err.println("Invalid number format: " + e.getMessage());
		}
		ScriptCommand cmd = new ScriptCommand("urcap_external_axis_jog_positive");
		cmd.appendLine(String.format(
			"axis_group_speedj(\"%s\", [%s], %s, 100)",
			groupAxisPairComboBox.getSelectedItem().toString().split("::")[0],
			Arrays.stream(velocities).mapToObj(df::format).collect(Collectors.joining(", ")),
			axisGroupAccelerationFactor.getText()));
		if (sender.executeURScript(cmd)) {
			axisGroupJogStatus.setIcon(axisGroupJogActiveIcon);
			axisGroupActiveCommand.setText(cmd.toString());
		}
	}

	private void stopAxisGroup() {
		double[] zeros = new double[selectedGroupSize];
		ScriptCommand cmd = new ScriptCommand("urcap_external_axis_jog_stop");
		cmd.appendLine(String.format(
			"axis_group_speedj(\"%s\", [%s], %s)",
			groupAxisPairComboBox.getSelectedItem().toString().split("::")[0],
			Arrays.stream(zeros).mapToObj(df::format).collect(Collectors.joining(", ")),
			axisGroupAccelerationFactor.getText()));
		if (sender.executeURScript(cmd)) {
			axisGroupJogStatus.setIcon(axisGroupJogInactiveIcon);
			axisGroupActiveCommand.setText(cmd.toString());
		}
	}

	// FIXME: The following script depends on 6 assumptions:
	// 1. The maximum number of axes is 6.
	// 2. The descendants of an axis_group are either a frame or an axis;
	// 3. The descendants of an axis are either a frame or an axis;
	// 4. The descendants of a frame (except "world") cannot be an axis;
	// 5. The parent of an axis_group can only be the "world" frame;
	// 6. The child of the "world" frame is either a frame or an axis_group.
	private String [] getGroupAxisPairs() {
		ScriptCommand cmd = new ScriptCommand("get_group_axis_pairs");
		cmd.appendLine(String.format("pairs = make_list(%d,\"\",%d)", MAXIMUM_NUMBER_AXES, MAXIMUM_NUMBER_AXES));
		// find all axes recursively
		cmd.appendLine("def get_axes_recursively(node, group):");
		cmd.appendLine("  axes = get_children(node)");
		cmd.appendLine("  iAxis = 0");
		cmd.appendLine("  while iAxis < length(axes):");
		cmd.appendLine("    axis = axes[iAxis]");
		cmd.appendLine("    if not is_frame(axis):");
		cmd.appendLine("      pairs[axis_group_get_axis_index(axis)] = group + \"::\" + axis");
		cmd.appendLine("      get_axes_recursively(axis, group)");
		cmd.appendLine("    end");
		cmd.appendLine("    iAxis = iAxis + 1");
		cmd.appendLine("  end");
		cmd.appendLine("end");
		// find all axis_group, then all group-axis pairs
		cmd.appendLine("groups = get_children(\"world\")");
		cmd.appendLine("iGroup = 0");
		cmd.appendLine("while iGroup < length(groups):");
		cmd.appendLine("  group = groups[iGroup]");
		cmd.appendLine("  if not is_frame(group):");
		cmd.appendLine("    get_axes_recursively(group, group)");
		cmd.appendLine("  end");
		cmd.appendLine("  iGroup = iGroup + 1");
		cmd.appendLine("end");
		// prepare and return result
		cmd.appendLine("retString = \"\"");
		cmd.appendLine("idx = 0");
		cmd.appendLine("while idx < length(pairs):");
		cmd.appendLine("  if pairs[idx] != \"\":");
		cmd.appendLine("    retString = retString + pairs[idx] + \",\"");
		cmd.appendLine("  end");
		cmd.appendLine("  idx = idx + 1");
		cmd.appendLine("end");
		cmd.appendLine(String.format("socket_open(\"127.0.0.1\", %d, \"return_socket\")", sender.RETURN_PORT));
		cmd.appendLine("socket_send_line(retString, \"return_socket\")");
		cmd.appendLine("socket_close(\"my_socket\")");
		return sender.executeURScriptWithStringReturn(cmd).split(",");
	}
}
