package com.ur.urcap.externalaxis.coordinatedmotionprogrammer.impl;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.ur.urcap.api.contribution.ContributionProvider;
import com.ur.urcap.api.contribution.ViewAPIProvider;
import com.ur.urcap.api.contribution.program.swing.SwingProgramNodeView;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardTextInput;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardInputFactory;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardNumberInput;

public class BaseMoveWithAxisGroupNodeView extends BaseProcessMoveNodeView {
  private final Style style;

  private KeyboardInputFactory keyboardInputFactory;

  private JTextField axisWaypointInputField = new JTextField();

  private JTextField robotWaypointInputField = new JTextField();

  public BaseMoveWithAxisGroupNodeView(Style style,
      KeyboardInputFactory keyboardInputFactory) {
    super(style, keyboardInputFactory);

    this.style = style;
    this.keyboardInputFactory = keyboardInputFactory;
  }

  public void setAxisWaypointInputField(String axisWaypoint) {
    axisWaypointInputField.setText(axisWaypoint);
  }

  public void setRobotWaypointInputField(String robotWaypoint) {
    robotWaypointInputField.setText(robotWaypoint);
  }

  // Creates a panel for capturing an axis waypoint. This includes a text box
  // for the waypoint and a button to capture the waypoint from its current
  // position.
  public <T extends BaseMoveWithAxisGroupNodeContribution> JPanel createAxisInputControls(
      final ContributionProvider<T> provider) {
    JPanel panel = new JPanel();
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
    panel.setAlignmentX(Component.LEFT_ALIGNMENT);

    JLabel paramsTitle = new JLabel("Axis Waypoint");
    paramsTitle.setFont(paramsTitle.getFont()
        .deriveFont((float) style.getSmallHeaderFontSize())
        .deriveFont(Font.BOLD));
    panel.add(paramsTitle);

    panel.add(createVerticalSpacing());
    panel.add(createLabelInputField(
        "Target:", axisWaypointInputField, new MouseAdapter() {
          @Override
          public void mousePressed(MouseEvent e) {
            KeyboardTextInput keyboard = getKeyboard(provider.get().getAxisWaypoint());
            keyboard.show(axisWaypointInputField,
                provider.get().getCallbackForAxisWaypoint());
          }
        }));

    panel.add(createVerticalSpacing());

    JButton teachWaypointButton = new JButton("Set to Current Position");

    teachWaypointButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        provider.get().teachAxisWaypoint();
      }
    });

    panel.add(teachWaypointButton);

    return panel;
  }

  // Creates a panel for capturing a robot waypoint. This includes a text box
  // for the waypoint and a button to capture the waypoint from the robot's
  // current position.
  public <T extends BaseMoveWithAxisGroupNodeContribution> JPanel createRobotInputControls(
      final ContributionProvider<T> provider) {
    JPanel panel = new JPanel();
    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
    panel.setAlignmentX(Component.LEFT_ALIGNMENT);

    JLabel paramsTitle = new JLabel("Robot Waypoint (in tracked frame)");
    paramsTitle.setFont(paramsTitle.getFont()
        .deriveFont((float) style.getSmallHeaderFontSize())
        .deriveFont(Font.BOLD));
    panel.add(paramsTitle);

    panel.add(createVerticalSpacing());
    panel.add(createLabelInputField(
        "Target:", robotWaypointInputField, new MouseAdapter() {
          @Override
          public void mousePressed(MouseEvent e) {
            KeyboardTextInput keyboard = getKeyboard(provider.get().getRobotWaypoint());
            keyboard.show(robotWaypointInputField,
                provider.get().getCallbackForRobotWaypoint());
          }
        }));

    panel.add(createVerticalSpacing());

    JButton teachWaypointButton = new JButton("Set to Current Position");

    teachWaypointButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        provider.get().teachRobotWaypoint();
      }
    });

    panel.add(teachWaypointButton);

    return panel;
  }
}
