package com.ur.urcap.externalaxis.coordinatedmotionprogrammer.impl;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.StringReader;
import java.net.Socket;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.concurrent.Callable;
import java.util.concurrent.FutureTask;
import java.net.ServerSocket;
import java.net.SocketTimeoutException;

import com.ur.urcap.api.contribution.ProgramNodeContribution;
import com.ur.urcap.api.contribution.program.ProgramAPIProvider;
import com.ur.urcap.api.domain.data.DataModel;
import com.ur.urcap.api.domain.script.ScriptWriter;
import com.ur.urcap.api.domain.undoredo.UndoRedoManager;
import com.ur.urcap.api.domain.undoredo.UndoableChanges;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardInputCallback;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardNumberInput;

public class MoveCWithAxisGroupNodeContribution extends
    BaseMoveWithAxisGroupNodeContribution implements ProgramNodeContribution,
    MoveCWithAxisGroupNodeInterface {

  private static final String VIA_AXIS_POSITION_KEY = "AXIS_VIA_POSITION"; // stores the axis position when the robot
                                                                           // via point
                                                                           // was captured for later use (e.g., to go
                                                                           // back to
                                                                           // the axis position when the via was taught
                                                                           // for
                                                                           // adjustment)

  private static final String VIA_POINT_KEY = "ROBOT_VIA_POINT"; // maps the robot via waypoint in the data model
                                                                 // stored as the TCP pose expressed in the tracked
                                                                 // frame and represented as a URScript pose, e.g.,
                                                                 // "p[0.1,0.1,0.1,0.1,0.1,0.1]"

  private final MoveCWithAxisGroupNodeView view;

  private final DataModel model;

  private final UndoRedoManager undoRedoManager;

  // This contains the implementation for the MoveCWithAxisGroup program node.
  // The details for getting the target pose and axis position are found in the
  // parent class. The details for getting the via robot position are in this
  // class.
  public MoveCWithAxisGroupNodeContribution(ProgramAPIProvider apiProvider,
      MoveCWithAxisGroupNodeView view,
      DataModel model) {
    super(apiProvider, view, model);

    this.view = view;
    this.model = model;
    this.undoRedoManager = apiProvider.getProgramAPI().getUndoRedoManager();
  }

  @Override
  public void openView() {
    super.openView();
  }

  @Override
  public void closeView() {
  }

  @Override
  public String getTitle() {
    return "MoveC with Axis Group";
  }

  // Indicates that the node is defined when axis group, axis waypoint, and
  // robot waypoints have been defined.
  @Override
  public boolean isDefined() {
    return super.isDefined() && !getRobotViaPoint().isEmpty();
  }

  // This is intentionally left empty because the CoordinatedProcessNode is in
  // charge of writing the script when the program is run or saved.
  @Override
  public void generateScript(ScriptWriter writer) {
  }

  void setRobotViaPoint(final String value) {
    view.setRobotViaPointInputField(value);

    undoRedoManager.recordChanges(new UndoableChanges() {
      @Override
      public void executeChanges() {
        model.set(VIA_POINT_KEY, value);
      }
    });
  }

  @Override
  public String getRobotViaPoint() {
    return model.get(VIA_POINT_KEY, "");
  }

  @Override
  public String getAxisGroup() {
    return super.getAxisGroup();
  }

  @Override
  public String getAxisWaypoint() {
    return super.getAxisWaypoint();
  }

  @Override
  public String getRobotWaypoint() {
    return super.getRobotWaypoint();
  }

  @Override
  public Double getSpeed() {
    return super.getSpeed();
  }

  @Override
  public Double getAcceleration() {
    return super.getAcceleration();
  }

  @Override
  public Double getBlendRadius() {
    return super.getBlendRadius();
  }

  // Stores the user keyboard input as the robot via point.
  public KeyboardInputCallback<String> getCallbackForRobotViaPoint() {
    return new KeyboardInputCallback<String>() {
      @Override
      public void onOk(final String value) {
        setRobotViaPoint(value);
      }
    };
  }

  // Retreives the current TCP pose in the tracked frame from the robot and
  // stores it as the movec via point.
  public void teachRobotViaPoint() {

    final String robotViaPoint = getStringResponseBackFromRobot(getRobotWaypointCaptureProgram());

    setRobotViaPoint(robotViaPoint);
  }
}
