package com.ur.urcap.externalaxis.ethercataxisconfig.installationNode;

import com.ur.urcap.api.contribution.InstallationNodeContribution;
import com.ur.urcap.api.contribution.installation.InstallationAPIProvider;
import com.ur.urcap.api.domain.data.DataModel;
import com.ur.urcap.api.domain.script.ScriptWriter;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardInputCallback;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardInputFactory;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardNumberInput;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardTextInput;

import com.ur.urcap.externalaxis.ethercataxisconfig.URCommunicator.ScriptSender;

import javax.swing.*;


public class EtherCATAxisConfigInstallationNodeContribution implements InstallationNodeContribution {

	private static final String AXIS_GROUP = "AxisGroup";
	private static final String AXIS_GROUP_DEFAULT = "ag1";

	private static final String AXIS_GROUP_POSE = "AxisGroupPose";
	private static final String AXIS_GROUP_POSE_DEFAULT = "p[0, 0, 0, 0, 0, 0]";

	private static final String AXIS = "Axis";
	private static final String AXIS_DEFAULT = "a1";

	private static final String AXIS_POSE = "AxisPose";
	private static final String AXIS_POSE_DEFAULT = "p[0, 0, 0, 0, 0, 0]";

	private static final String AXIS_TYPE = "AxisType";
	private static final String AXIS_TYPE_DEFAULT = "Rotary";

	private static final String MAX_VELOCITY = "MaxVelocity";
	private static final String MAX_ACCELERATION = "MaxAcceleration";
	private static final String MIN = "Min";
	private static final String MAX = "Max";
	private static final String COUNTS_PER_REV = "CountsPerRev";
	private static final String GEAR_RATIO = "GearRatio";
	private static final String FEED_CONSTANT = "FeedConstant";
	private static final String ZERO_OFFSET = "ZeroOffset";
	private static final String CHECKBOX = "CheckBox";

	private final EtherCATAxisConfigInstallationNodeView view;
	private final DataModel model;
	private final KeyboardInputFactory keyboardInputFactory;

	private final ScriptSender sender;


	public EtherCATAxisConfigInstallationNodeContribution(InstallationAPIProvider apiProvider,
														  EtherCATAxisConfigInstallationNodeView view,
														  DataModel model) {
		this.view = view;
		this.model = model;
		this.keyboardInputFactory = apiProvider.getUserInterfaceAPI().getUserInteraction().getKeyboardInputFactory();
		sender = view.getSender();

	}

	@Override
	public void openView() {
		sender.connect("127.0.0.1", ScriptSender.PRIMARY_PORT, 200);
		view.setAxisGroupName(getAxisGroup());
		view.setAxisGroupPose(getAxisGroupPose());
		view.setAxisName(getAxis());
		view.setAxisPose(getAxisPose());

		String storedAxisType = model.get(AXIS_TYPE, AXIS_TYPE_DEFAULT);
		view.setSelectedAxisType(storedAxisType);

		Boolean storedCheckBoxState = model.get(CHECKBOX, false);
		view.setCheckBoxState(storedCheckBoxState);

		view.setAxisVelocityLimit(getMaxVelocity());
		view.setAxisAccelerationLimit(getMaxAcceleration());
		view.setAxisPositionLimitMin(getMin());
		view.setAxisPositionLimitMax(getMax());
		view.setEtherCATCountsPerRevolution(getCountsPerRev());
		view.setEtherCATGearRatio(getGearRatio());
		view.setEtherCATFeedConstant(getFeedConstant());
		view.setEtherCATZeroOffset(getZeroOffset());

	}

	@Override
	public void closeView() {
		sender.disconnect();
	}

	@Override
	public void generateScript(ScriptWriter writer) {
	}


	public KeyboardTextInput getKeyboardForAxisGroup() {
		KeyboardTextInput keyboard = keyboardInputFactory.createStringKeyboardInput();
		keyboard.setInitialValue(model.get(AXIS_GROUP, AXIS_GROUP_DEFAULT));
		return keyboard;
	}
	public KeyboardInputCallback<String> getCallbackForAxisGroup() {
		return new KeyboardInputCallback<String>() {
			@Override
			public void onOk(String value) {
				model.set(AXIS_GROUP, value);
				view.setAxisGroupName(value);
			}
		};
	}
	private String getAxisGroup() {
		return model.get(AXIS_GROUP, AXIS_GROUP_DEFAULT);
	}


	public KeyboardTextInput getKeyboardForAxisGroupPose() {
		KeyboardTextInput keyboard = keyboardInputFactory.createStringKeyboardInput();
		keyboard.setInitialValue(model.get(AXIS_GROUP_POSE, AXIS_GROUP_POSE_DEFAULT));
		return keyboard;
	}
	public KeyboardInputCallback<String> getCallbackForAxisGroupPose() {
		return new KeyboardInputCallback<String>() {
			@Override
			public void onOk(String value) {
				model.set(AXIS_GROUP_POSE, value);
				view.setAxisGroupPose(value);
			}
		};
	}
	private String getAxisGroupPose() {
		return model.get(AXIS_GROUP_POSE, AXIS_GROUP_POSE_DEFAULT);
	}


	public KeyboardTextInput getKeyboardForAxis() {
		KeyboardTextInput keyboard = keyboardInputFactory.createStringKeyboardInput();
		keyboard.setInitialValue(model.get(AXIS, AXIS_DEFAULT));
		return keyboard;
	}
	public KeyboardInputCallback<String> getCallbackForAxis() {
		return new KeyboardInputCallback<String>() {
			@Override
			public void onOk(String value) {
				model.set(AXIS, value);
				view.setAxisName(value);
			}
		};
	}
	private String getAxis() {
		return model.get(AXIS, AXIS_DEFAULT);
	}


	public KeyboardTextInput getKeyboardForAxisPose() {
		KeyboardTextInput keyboard = keyboardInputFactory.createStringKeyboardInput();
		keyboard.setInitialValue(model.get(AXIS_POSE, AXIS_POSE_DEFAULT));
		return keyboard;
	}
	public KeyboardInputCallback<String> getCallbackForAxisPose() {
		return new KeyboardInputCallback<String>() {
			@Override
			public void onOk(String value) {
				model.set(AXIS_POSE, value);
				view.setAxisPose(value);
			}
		};
	}
	private String getAxisPose() {
		return model.get(AXIS_POSE, AXIS_POSE_DEFAULT);
	}


	public void AxisTypeCallback(String value) {
		model.set(AXIS_TYPE, value);

	}

	public void CheckBoxCallback(Boolean state) {
		model.set(CHECKBOX, state);
		view.setCheckBoxState(state);
	}

	public KeyboardNumberInput<Double> getKeyboardForMaxVelocity() {
		KeyboardNumberInput<Double> keyboard = keyboardInputFactory.createDoubleKeypadInput();
		keyboard.setInitialValue(model.get(MAX_VELOCITY, 0.0D));
		return keyboard;
	}
	public KeyboardInputCallback<Double> getCallbackForMaxVelocity() {
		return new KeyboardInputCallback<Double>() {
			@Override
			public void onOk(Double value) {
				model.set(MAX_VELOCITY, value);
				view.setAxisVelocityLimit(value);
			}
		};
	}
	private double getMaxVelocity() {
		return model.get(MAX_VELOCITY, 0.0D);
	}


	public KeyboardNumberInput<Double> getKeyboardForMaxAcceleration() {
		KeyboardNumberInput<Double> keyboard = keyboardInputFactory.createDoubleKeypadInput();
		keyboard.setInitialValue(model.get(MAX_ACCELERATION, 0.0D));
		return keyboard;
	}
	public KeyboardInputCallback<Double> getCallbackForMaxAcceleration() {
		return new KeyboardInputCallback<Double>() {
			@Override
			public void onOk(Double value) {
				model.set(MAX_ACCELERATION, value);
				view.setAxisAccelerationLimit(value);
			}
		};
	}
	private double getMaxAcceleration() {
		return model.get(MAX_ACCELERATION, 0.0D);
	}


	public KeyboardNumberInput<Double> getKeyboardForMin() {
		KeyboardNumberInput<Double> keyboard = keyboardInputFactory.createDoubleKeypadInput();
		keyboard.setInitialValue(model.get(MIN, 0.0D));
		return keyboard;
	}
	public KeyboardInputCallback<Double> getCallbackForMin() {
		return new KeyboardInputCallback<Double>() {
			@Override
			public void onOk(Double value) {
				model.set(MIN, value);
				view.setAxisPositionLimitMin(value);
			}
		};
	}
	private double getMin() {
		return model.get(MIN, 0.0D);
	}


	public KeyboardNumberInput<Double> getKeyboardForMax() {
		KeyboardNumberInput<Double> keyboard = keyboardInputFactory.createDoubleKeypadInput();
		keyboard.setInitialValue(model.get(MAX, 0.0D));
		return keyboard;
	}
	public KeyboardInputCallback<Double> getCallbackForMax() {
		return new KeyboardInputCallback<Double>() {
			@Override
			public void onOk(Double value) {
				model.set(MAX, value);
				view.setAxisPositionLimitMax(value);
			}
		};
	}
	private double getMax() {
		return model.get(MAX, 0.0D);
	}


	public KeyboardNumberInput<Integer> getKeyboardForCountsPerRev() {
		KeyboardNumberInput<Integer> keyboard = keyboardInputFactory.createPositiveIntegerKeypadInput();
		keyboard.setInitialValue(model.get(COUNTS_PER_REV, 1));
		return keyboard;
	}
	public KeyboardInputCallback<Integer> getCallbackForCountsPerRev() {
		return new KeyboardInputCallback<Integer>() {
			@Override
			public void onOk(Integer value) {
				if (value > 0){
					model.set(COUNTS_PER_REV, value);
					view.setEtherCATCountsPerRevolution(value);
				}
				else {
					JOptionPane.showMessageDialog(null, "Counts per revolution must be greater than 0", "Error", JOptionPane.ERROR_MESSAGE);
				}

			}
		};
	}
	private int getCountsPerRev() {
		return model.get(COUNTS_PER_REV, 1);
	}


	public KeyboardNumberInput<Double> getKeyboardForGearRatio() {
		KeyboardNumberInput<Double> keyboard = keyboardInputFactory.createDoubleKeypadInput();
		keyboard.setInitialValue(model.get(GEAR_RATIO, 1.0D));
		return keyboard;
	}
	public KeyboardInputCallback<Double> getCallbackForGearRatio() {
		return new KeyboardInputCallback<Double>() {
			@Override
			public void onOk(Double value) {
				if (value > 0){
					model.set(GEAR_RATIO, value);
					view.setEtherCATGearRatio(value);
				}
				else {
					JOptionPane.showMessageDialog(null, "Gear ratio must be greater than 0", "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		};
	}
	private double getGearRatio() {
		return model.get(GEAR_RATIO, 1.0D);
	}


	public KeyboardNumberInput<Double> getKeyboardForFeedConstant() {
		KeyboardNumberInput<Double> keyboard = keyboardInputFactory.createDoubleKeypadInput();
		keyboard.setInitialValue(model.get(FEED_CONSTANT, 6.28318530718D));
		return keyboard;
	}
	public KeyboardInputCallback<Double> getCallbackForFeedConstant() {
		return new KeyboardInputCallback<Double>() {
			@Override
			public void onOk(Double value) {
				if (value != 0){
					model.set(FEED_CONSTANT, value);
					view.setEtherCATFeedConstant(value);
				}
				else {
					JOptionPane.showMessageDialog(null, "Feed constant cannot be 0", "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		};
	}
	private double getFeedConstant() {
		return model.get(FEED_CONSTANT, 6.28318530718D);
	}


	public KeyboardNumberInput<Integer> getKeyboardForZeroOffset() {
		KeyboardNumberInput<Integer> keyboard = keyboardInputFactory.createPositiveIntegerKeypadInput();
		keyboard.setInitialValue(model.get(ZERO_OFFSET, 1));
		return keyboard;
	}
	public KeyboardInputCallback<Integer> getCallbackZeroOffset() {
		return new KeyboardInputCallback<Integer>() {
			@Override
			public void onOk(Integer value) {
				model.set(ZERO_OFFSET, value);
				view.setEtherCATZeroOffset(value);
			}
		};
	}
	private int getZeroOffset() {
		return model.get(ZERO_OFFSET, 0);
	}

}
