package com.ur.urcap.externalaxis.ethercataxisconfig.installationNode;

import com.ur.urcap.api.contribution.installation.swing.SwingInstallationNodeView;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardNumberInput;
import com.ur.urcap.api.domain.userinteraction.keyboard.KeyboardTextInput;

import javax.swing.*;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import com.ur.urcap.externalaxis.ethercataxisconfig.impl.Style;
import com.ur.urcap.externalaxis.ethercataxisconfig.URCommunicator.ScriptCommand;
import com.ur.urcap.externalaxis.ethercataxisconfig.URCommunicator.ScriptSender;


public class EtherCATAxisConfigInstallationNodeView implements SwingInstallationNodeView<EtherCATAxisConfigInstallationNodeContribution> {

	private static final int MAXIMUM_FRACTION_DIGITS = 8;
	private final Style style;

	private final JTextField axisGroupName = new JTextField();
	private final JTextField axisGroupPose = new JTextField();
	private final JTextField axisName = new JTextField();
	private final JTextField axisPose = new JTextField();
	private final JCheckBox axisPositionLimitsEnabled = new JCheckBox("", false);
	private final JLabel minLabel = new JLabel("Min:");
	private final JLabel maxLabel = new JLabel("Max:");
 	private final JTextField axisVelocityLimit = new JTextField();
	private final JTextField axisAccelerationLimit = new JTextField();
	private final JTextField axisPositionLimitMin = new JTextField();
	private final JTextField axisPositionLimitMax = new JTextField();
	private final JTextField etherCATCountsPerRevolution = new JTextField();
	private final JTextField etherCATGearRatio = new JTextField();
	private final JTextField etherCATFeedConstant = new JTextField();
	private final JTextField etherCATZeroOffset = new JTextField();

	private int selectedAxisType = 0;
	private final JRadioButton axisTypeRotary = new JRadioButton("Rotary", true);
	private final JRadioButton axisTypeLinear = new JRadioButton("Linear", false);
	private final ButtonGroup buttonGroup= new ButtonGroup();

	private final JButton sendScriptButton = new JButton("Apply");
	private final ScriptSender sender;

	public EtherCATAxisConfigInstallationNodeView(Style style) {
		this.style = style;
		sender = new ScriptSender();
	}

	@Override
	public void buildUI(JPanel jPanel, final EtherCATAxisConfigInstallationNodeContribution contribution) {

		axisTypeRotary.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				contribution.AxisTypeCallback("Rotary");
			}
		});

		axisTypeLinear.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				contribution.AxisTypeCallback("Linear");
			}

		});

		axisPositionLimitsEnabled.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				contribution.CheckBoxCallback(axisPositionLimitsEnabled.isSelected());
			}
		});

		buttonGroup.add(axisTypeRotary);
		buttonGroup.add(axisTypeLinear);
		jPanel.add(axisTypeRotary);
		jPanel.add(axisTypeLinear);

		JLabel axisGroupLabel = new JLabel("Axis Group:");
		setupTextInputField(axisGroupName, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardTextInput keyboard = contribution.getKeyboardForAxisGroup();
				keyboard.show(axisGroupName, contribution.getCallbackForAxisGroup());
			}
		});

		JLabel axisPoseLabel = new JLabel("Axis Pose:");
		setupTextInputField(axisPose, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardTextInput keyboard = contribution.getKeyboardForAxisPose();
				keyboard.show(axisPose, contribution.getCallbackForAxisPose());
			}
		});

		JLabel axisGroupPoseLabel = new JLabel("Axis Group Pose:");
		setupTextInputField(axisGroupPose, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardTextInput keyboard = contribution.getKeyboardForAxisGroupPose();
				keyboard.show(axisGroupPose, contribution.getCallbackForAxisGroupPose());
			}
		});

		JLabel axisLabel = new JLabel("Axis:");
		setupTextInputField(axisName, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardTextInput keyboard = contribution.getKeyboardForAxis();
				keyboard.show(axisName, contribution.getCallbackForAxis());
			}
		});

		JLabel axisTypeLabel = new JLabel("Axis Type:");

		JLabel maxVelocityLabel = new JLabel("Max Velocity:");
		setupTextInputField(axisVelocityLimit, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardNumberInput<Double> keyboard = contribution.getKeyboardForMaxVelocity();
				keyboard.show(axisVelocityLimit, contribution.getCallbackForMaxVelocity());
			}
		});

		JLabel maxAccelerationLabel = new JLabel("Max Acceleration:");
		setupTextInputField(axisAccelerationLimit, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardNumberInput<Double> keyboard = contribution.getKeyboardForMaxAcceleration();
				keyboard.show(axisAccelerationLimit, contribution.getCallbackForMaxAcceleration());
			}
		});

		JLabel axisPositionLimitsEnabledLabel = new JLabel("Position Limits:");

		minLabel.setEnabled(false);
		axisPositionLimitMin.setEnabled(false);
		setupTextInputField(axisPositionLimitMin, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardNumberInput<Double> keyboard = contribution.getKeyboardForMin();
				keyboard.show(axisPositionLimitMin, contribution.getCallbackForMin());
			}
		});

		maxLabel.setEnabled(false);
		axisPositionLimitMax.setEnabled(false);
		setupTextInputField(axisPositionLimitMax, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardNumberInput<Double> keyboard = contribution.getKeyboardForMax();
				keyboard.show(axisPositionLimitMax, contribution.getCallbackForMax());
			}
		});

		JLabel countsPerRevLabel = new JLabel("Counts per Rev:");
		setupTextInputField(etherCATCountsPerRevolution, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardNumberInput<Integer> keyboard = contribution.getKeyboardForCountsPerRev();
				keyboard.show(etherCATCountsPerRevolution, contribution.getCallbackForCountsPerRev());
			}
		});

		JLabel gearRatioLabel = new JLabel("Gear Ratio:");
		setupTextInputField(etherCATGearRatio, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardNumberInput<Double> keyboard = contribution.getKeyboardForGearRatio();
				keyboard.show(etherCATGearRatio, contribution.getCallbackForGearRatio());
			}
		});

		JLabel feedConstantLabel = new JLabel("Feed Constant:");
		setupTextInputField(etherCATFeedConstant, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardNumberInput<Double> keyboard = contribution.getKeyboardForFeedConstant();
				keyboard.show(etherCATFeedConstant, contribution.getCallbackForFeedConstant());
			}
		});

		JLabel zeroOffsetLabel = new JLabel("Zero Offset:");
		setupTextInputField(etherCATZeroOffset, new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				KeyboardNumberInput<Integer> keyboard = contribution.getKeyboardForZeroOffset();
				keyboard.show(etherCATZeroOffset, contribution.getCallbackZeroOffset());
			}
		});


		sendScriptButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				sendScript();
			}
		});


		GroupLayout groupLayout = new GroupLayout(jPanel);
		jPanel.setLayout(groupLayout);

		groupLayout.setAutoCreateGaps(true);
		groupLayout.setAutoCreateContainerGaps(true);

		Component verticalSpacing = createVerticalSpacing();
		Component horizontalSpacing = createHorizontalSpacing();

		final int boxSize = 175;

		groupLayout.setHorizontalGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addGroup(groupLayout.createSequentialGroup()
						.addGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
								.addComponent(axisGroupLabel)
								.addComponent(axisLabel)
								.addComponent(axisTypeLabel)
								.addComponent(maxVelocityLabel)
								.addComponent(axisPositionLimitsEnabledLabel)
								.addComponent(minLabel)
								.addComponent(countsPerRevLabel)
								.addComponent(feedConstantLabel))
						.addComponent(verticalSpacing)
						.addComponent(horizontalSpacing)
						.addGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
								.addComponent(axisGroupName,boxSize, boxSize, boxSize)
								.addComponent(axisName,boxSize, boxSize, boxSize)
								.addComponent(axisTypeRotary)
								.addComponent(axisVelocityLimit,boxSize, boxSize, boxSize)
								.addComponent(axisPositionLimitsEnabled,boxSize, boxSize, boxSize)
								.addComponent(axisPositionLimitMin,boxSize, boxSize, boxSize)
								.addComponent(etherCATCountsPerRevolution,boxSize, boxSize, boxSize)
								.addComponent(etherCATFeedConstant,boxSize, boxSize, boxSize))
						.addComponent(verticalSpacing)
						.addComponent(horizontalSpacing)
						.addComponent(horizontalSpacing)
						.addGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
								.addComponent(axisGroupPoseLabel)
								.addComponent(axisPoseLabel)
								.addComponent(maxAccelerationLabel)
								.addComponent(maxLabel)
								.addComponent(gearRatioLabel)
								.addComponent(zeroOffsetLabel))
						.addComponent(verticalSpacing)
						.addComponent(horizontalSpacing)
						.addGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.LEADING)
								.addComponent(axisGroupPose,boxSize, boxSize, boxSize)
								.addComponent(axisPose,boxSize, boxSize, boxSize)
								.addComponent(axisTypeLinear)
								.addComponent(axisAccelerationLimit,boxSize, boxSize, boxSize)
								.addComponent(axisPositionLimitMax,boxSize, boxSize, boxSize)
								.addComponent(etherCATGearRatio,boxSize, boxSize, boxSize)
								.addComponent(etherCATZeroOffset,boxSize, boxSize, boxSize)))
				.addComponent(verticalSpacing)
				.addComponent(verticalSpacing)
				.addComponent(sendScriptButton));

		groupLayout.setVerticalGroup(groupLayout.createSequentialGroup()
				.addGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(axisGroupLabel)
						.addComponent(axisGroupName)
						.addComponent(axisGroupPoseLabel)
						.addComponent(axisGroupPose))
				.addComponent(verticalSpacing)
				.addComponent(horizontalSpacing)
						.addGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(axisLabel)
						.addComponent(axisName)
						.addComponent(axisPoseLabel)
						.addComponent(axisPose))
				.addComponent(verticalSpacing)
				.addComponent(horizontalSpacing)
				.addComponent(horizontalSpacing)
						.addGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(axisTypeLabel)
						.addComponent(axisTypeRotary)
						.addComponent(axisTypeLinear))
				.addComponent(verticalSpacing)
				.addComponent(horizontalSpacing)
						.addGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(maxVelocityLabel)
						.addComponent(axisVelocityLimit)
						.addComponent(maxAccelerationLabel)
						.addComponent(axisAccelerationLimit))
				.addComponent(verticalSpacing)

				.addGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(countsPerRevLabel)
						.addComponent(etherCATCountsPerRevolution)
						.addComponent(gearRatioLabel)
						.addComponent(etherCATGearRatio))
				.addComponent(verticalSpacing)

				.addGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(feedConstantLabel)
						.addComponent(etherCATFeedConstant)
						.addComponent(zeroOffsetLabel)
						.addComponent(etherCATZeroOffset))
				.addComponent(verticalSpacing)

				.addGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(axisPositionLimitsEnabledLabel)
						.addComponent(axisPositionLimitsEnabled))
				.addComponent(verticalSpacing)

				.addGroup(groupLayout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(minLabel)
						.addComponent(axisPositionLimitMin)
						.addComponent(maxLabel)
						.addComponent(axisPositionLimitMax))
				.addComponent(verticalSpacing)


				.addComponent(verticalSpacing)
				.addComponent(sendScriptButton));
	}


	private void setupTextInputField(final JTextField inputField, MouseAdapter mouseAdapter) {
		inputField.setFocusable(false);
		inputField.setPreferredSize(style.getInputfieldDimension());
		inputField.setMaximumSize(inputField.getPreferredSize());
		inputField.addMouseListener(mouseAdapter);
	}

	private Component createVerticalSpacing() {
		return Box.createRigidArea(new Dimension(0, style.getVerticalSpacing()));
	}
	private Component createHorizontalSpacing() {
		return Box.createRigidArea(new Dimension(style.getHorizontalSpacing(), 0));
	}


	public void setAxisGroupName(String value) {
		axisGroupName.setText(value);
	}

	public void setAxisGroupPose(String value) {
		axisGroupPose.setText(value);
	}

	public void setAxisName(String value) {
		axisName.setText(value);
	}

	public void setAxisPose(String value) {
		axisPose.setText(value);
	}

	public void setSelectedAxisType(String storedAxisType) {
		if (storedAxisType.equals("Rotary")) {
			axisTypeRotary.setSelected(true);
			this.selectedAxisType = 0;
		} else if (storedAxisType.equals("Linear")) {
			axisTypeLinear.setSelected(true);
			this.selectedAxisType = 1;
		}
	}

	public void setCheckBoxState(Boolean checkBoxState) {
        axisPositionLimitsEnabled.setSelected(checkBoxState);
		minLabel.setEnabled(checkBoxState);
		maxLabel.setEnabled(checkBoxState);
		axisPositionLimitMin.setEnabled(checkBoxState);
		axisPositionLimitMax.setEnabled(checkBoxState);
	}

	public void setAxisVelocityLimit(Double value) {
		DecimalFormat df = new DecimalFormat("#");
		df.setMaximumFractionDigits(MAXIMUM_FRACTION_DIGITS);
		String stringValue = df.format((double) value);
		axisVelocityLimit.setText(stringValue);
	}

	public void setAxisAccelerationLimit(Double value) {
		DecimalFormat df = new DecimalFormat("#");
		df.setMaximumFractionDigits(MAXIMUM_FRACTION_DIGITS);
		String stringValue = df.format((double) value);
		axisAccelerationLimit.setText(stringValue);
	}

	public void setAxisPositionLimitMin(Double value) {
		DecimalFormat df = new DecimalFormat("#");
		df.setMaximumFractionDigits(MAXIMUM_FRACTION_DIGITS);
		String stringValue = df.format((double) value);
		axisPositionLimitMin.setText(stringValue);
	}

	public void setAxisPositionLimitMax(Double value) {
		DecimalFormat df = new DecimalFormat("#");
		df.setMaximumFractionDigits(MAXIMUM_FRACTION_DIGITS);
		String stringValue = df.format((double) value);
		axisPositionLimitMax.setText(stringValue);
	}

	public void setEtherCATCountsPerRevolution(Integer value) {
		String stringValue = String.valueOf(value);
		etherCATCountsPerRevolution.setText(stringValue);
	}

	public void setEtherCATGearRatio(Double value) {
		DecimalFormat df = new DecimalFormat("#");
		df.setMaximumFractionDigits(MAXIMUM_FRACTION_DIGITS);
		String stringValue = df.format((double) value);
		etherCATGearRatio.setText(stringValue);
	}

	public void setEtherCATFeedConstant(Double value) {
		DecimalFormat df = new DecimalFormat("#");
		df.setMaximumFractionDigits(MAXIMUM_FRACTION_DIGITS);
		String stringValue = df.format((double) value);
		etherCATFeedConstant.setText(stringValue);
	}

	public void setEtherCATZeroOffset(Integer value) {
		String stringValue = String.valueOf(value);
		etherCATZeroOffset.setText(stringValue);
	}

	private void sendScript() {
		// Construct the primary program to be sent to the controller
		try {
			// Grab the EtherCAT API definitions from the filesystem
			String preambleFileContents = new String(
					Files.readAllBytes(Paths.get("/usr/local/motionplus/ethercat_definitions.script")));

			ScriptCommand cmd = new ScriptCommand("urcap_external_axis_ethercat_config_axis");

			BufferedReader bufReader = new BufferedReader(new StringReader(preambleFileContents));
			String line = null;
			while ((line = bufReader.readLine()) != null) {
				cmd.appendLine(line);
			}
			cmd.appendLine("ethercat_clear_error()");
			cmd.appendLine("ethercat_stop(True)");
			cmd.appendLine("reset_world_model()");
			cmd.appendLine(
					String.format("axis_group_add(\"%s\", %s, \"base\")", axisGroupName.getText(), axisGroupPose.getText()));

			String axisPositionLimits = "";
			String formatString = "axis_group_add_axis(\"%s\", \"%s\", \"\", %s, %s, %s, %s" +
					(axisPositionLimitsEnabled.isSelected() ? ", %s" : "") + ")";

			if (axisPositionLimitsEnabled.isSelected()) {
				axisPositionLimits = String.format("[%s, %s]",
						axisPositionLimitMin.getText(),
						axisPositionLimitMax.getText()
				);
			}

			cmd.appendLine(String.format(
					formatString,
					axisGroupName.getText(), axisName.getText(),
					axisPose.getText(), this.selectedAxisType,
					axisVelocityLimit.getText(), axisAccelerationLimit.getText(),
					axisPositionLimits
			));

			cmd.appendLine(String.format(
					"ethercat_config_axis(\"%s\", 1, %s, %s, %s, %s)",
					axisName.getText(), etherCATCountsPerRevolution.getText(),
					etherCATGearRatio.getText(), etherCATFeedConstant.getText(),
					etherCATZeroOffset.getText()
			));

			cmd.appendLine("ethercat_set_parameter(\"dc_enable\", True)");
			cmd.appendLine("ethercat_start(10)");
			cmd.appendLine(String.format("if (ethercat_is_axis_in_fault_state(\"%s\")):", axisName.getText()));
			cmd.appendLine(String.format("  textmsg(\"FAULT in: \", \"%s\")", axisName.getText()));
			cmd.appendLine(String.format("  if(ethercat_reset_axis_fault(\"%s\")):", axisName.getText()));
			cmd.appendLine("    textmsg(\"FAULT cleared\")");
			cmd.appendLine("  end");
			cmd.appendLine("end");
			cmd.appendLine(String.format("ethercat_enable_axis(\"%s\")", axisName.getText()));
			cmd.appendLine("popup(\"Axis successfully configured!\", \"Axis Configuration Status\", False)");

			if (!sender.executeURScript(cmd)) {
				throw new IOException();
			}

		} catch (IOException e) {
			System.err.println("Failed to construct the primary program.");
			return;
		}
	}

	public ScriptSender getSender() {
		return sender;
	}
}
