package com.ur.urcap.externalaxis.linearaxiscalibration.installationNode.installationNodeContribution;

import com.ur.urcap.api.contribution.InstallationNodeContribution;
import com.ur.urcap.api.domain.script.ScriptWriter;
import com.ur.urcap.externalaxis.linearaxiscalibration.URCommunicator.ScriptSender;
import com.ur.urcap.externalaxis.linearaxiscalibration.installationNode.installationNodeContribution.interfaces.SamplePoseButtonListener;
import com.ur.urcap.externalaxis.linearaxiscalibration.installationNode.installationNodeView.LinearAxisCalibrationInstallationNodeView;
import com.ur.urcap.externalaxis.linearaxiscalibration.installationNode.installationNodeView.SamplePosePanel;

import java.io.DataInputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.Arrays;


public class LinearAxisCalibrationInstallationNodeContribution implements InstallationNodeContribution, SamplePoseButtonListener {

	private final ScriptSender sender;
	private final SamplePosePanel samplePosePanel;

	public LinearAxisCalibrationInstallationNodeContribution(LinearAxisCalibrationInstallationNodeView view) {
		sender = new ScriptSender();
		sender.connect("127.0.0.1", ScriptSender.PRIMARY_PORT, 200);

		samplePosePanel = view.getSamplePosePanel();
		samplePosePanel.setButtonClickListener(this);
	}


	private enum RTinfo {
		TCP_actual		(56, 6);

		private final int index;
		private final int count;
		RTinfo(int index, int count){
			this.index = index;
			this.count = count;
		}
		private int index() {return index;}
		private int count() {return count;}
	}


	@Override
	public void onSamplePoseButtonClicked() {
		/**
		 * Invoked when the 'Sample Pose' button is clicked.
		 * This method captures the current TCP pose in the base frame using RTDE.
		 * The captured pose is then added to a collection of any previously sampled poses and displayed.
		 */
		double[] RealtimeMessage;

		RealtimeMessage = readRTDESocket();

		double[] val = new double[RTinfo.TCP_actual.count()];
		int i = 0;
		while (i < RTinfo.TCP_actual.count()){
			val[i] = RealtimeMessage[RTinfo.TCP_actual.index()+i];
			++i;
		}
		samplePosePanel.setLabelText(Arrays.toString(val));
	}


	private double[] readRTDESocket(){
		/**
		 * This method establishes a socket connection using RTDE and reads data from the stream.
		 * The data is then stored in the RealtimeMessage array.
		 * The method returns the RealtimeMessage array for further processing.
		 */
		double[] RealtimeMessage = null;

		try{
			// Create a new Socket Client
			Socket rt = new Socket("127.0.0.1", 30003);
			if (rt.isConnected()){
				System.out.println("Connected to UR Realtime Client");
			}

			// Create stream for data
			DataInputStream in;
			in = new DataInputStream(rt.getInputStream());

			// Read the integer available in stream
			int length = in.readInt();

			// Initialize size of RealtimeMessage be using received length
			RealtimeMessage = new double[length];
			// Add length integer to output array
			RealtimeMessage[0] = length;

			// Calculate how much data is available from the length
			int data_available = (length-4)/8;

			// Loop over reading the available data
			int i = 1;
			while (i <= data_available){
				RealtimeMessage[i] = in.readDouble();
				i++;
			}

			// Perform housekeeping
			in.close();
			rt.close();
			System.out.println("Disconnected from UR Realtime Client");
		}
		catch (IOException e){
			System.out.println(e);
		}
		return RealtimeMessage;
	}

	@Override
	public void openView() {
	}

	@Override
	public void closeView() {
	}

	@Override
	public void generateScript(ScriptWriter writer) {
	}
}
