package com.ur.urcap.externalaxis.linearaxiscalibration.installationNode.installationNodeView;

import com.ur.urcap.externalaxis.linearaxiscalibration.URCommunicator.ScriptCommand;
import com.ur.urcap.externalaxis.linearaxiscalibration.URCommunicator.ScriptSender;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;

public class CalibrationResultPanel extends JPanel {
    private final JLabel calibrationResultLabel;
    private final JLabel calibrationCommandLabel;

    private final LinearAxisCalibrationInstallationNodeView view;
    private final JLabel noteLabel;

    private JButton calibrateAxisButton = new JButton();

    private final ScriptSender sender;
    private String isMounted;

    public CalibrationResultPanel(LinearAxisCalibrationInstallationNodeView view) {
        this.view = view;
        sender = new ScriptSender();

        calibrateAxisButton.setText("Calibrate Axis");
        calibrateAxisButton.setEnabled(false);
        calibrateAxisButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                calibrateAxisButtonClicked(evt);

            }
        });

        JLabel calibrationCommandLabel = new JLabel("<html><b><u>URScript Command:</u></b></html>");
        JLabel calibrationResultLabel = new JLabel("<html><b><u>Calibration Result:</u></b></html>");

        this.calibrationCommandLabel = new JLabel("No data yet");
        this.calibrationResultLabel = new JLabel("No data yet");
        this.noteLabel = new JLabel("");

        JButton copyPoseButton = new JButton("Copy pose to clipboard");
        copyPoseButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                try {
                    copyPoseMouseClicked(evt);
                } catch (IOException e) {
                    throw new RuntimeException(e);
                } catch (UnsupportedFlavorException e) {
                    throw new RuntimeException(e);
                }
            }
        });

        int width = 480;
        int height = 200;
        JPanel scriptCommandPanel = new JPanel();
        BoxLayout boxLayout = new BoxLayout(scriptCommandPanel, BoxLayout.Y_AXIS);

        scriptCommandPanel.setPreferredSize(new Dimension(width, height));
        scriptCommandPanel.setMaximumSize(new Dimension(width, height));
        scriptCommandPanel.setLayout(boxLayout);

        scriptCommandPanel.add(Box.createRigidArea(new Dimension(0, 20)));
        scriptCommandPanel.add(calibrationCommandLabel);
        scriptCommandPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        scriptCommandPanel.add(this.calibrationCommandLabel);
        add(scriptCommandPanel);

        JPanel resultPanel = new JPanel();
        BoxLayout boxResultPanel = new BoxLayout(resultPanel, BoxLayout.Y_AXIS);

        resultPanel.setPreferredSize(new Dimension(width, 282));
        resultPanel.setMaximumSize(new Dimension(width, 282));
        resultPanel.setLayout(boxResultPanel);

        resultPanel.add(calibrationResultLabel);
        resultPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        resultPanel.add(this.calibrationResultLabel);
        resultPanel.add(Box.createRigidArea(new Dimension(0, 30)));
        resultPanel.add(this.noteLabel);
        add(resultPanel);

        JPanel boxPanel = new JPanel();
        boxPanel.add(Box.createRigidArea(new Dimension(0, 150)));
        Box horizontalBox = Box.createHorizontalBox();
        horizontalBox.add(calibrateAxisButton);
        horizontalBox.add(Box.createRigidArea(new Dimension(40, 0)));
        horizontalBox.add(copyPoseButton);
        boxPanel.add(horizontalBox);
        add(boxPanel);
    }

    public void setUrsriptCommandLabel(String calibrationCommand) {
        /** Formats the URScript command string in order to be properly displayed.
         * The string is split after each comma.
         */

        calibrationCommand = "<html>" + calibrationCommand.replace("p", "<br>p") + "</html>";
        calibrationCommand = calibrationCommand.replaceAll("(\\d+\\.\\d{4})\\d+", "$1");

        calibrationCommandLabel.setText(calibrationCommand);
    }

    public void setCalibrationResult(String calibrationResult) {
        /** Formats the calibration result string in order to be properly displayed.
         * The string is split after each comma except for the line containing the pose.
         */

        calibrationResult = calibrationResult.replaceAll("(\\d+\\.\\d{4})\\d+", "$1");

        String[] parts = calibrationResult.split(", ");

        StringBuilder formattedString = new StringBuilder("<html>");
        for (int i = 0; i < parts.length; i++) {
            formattedString.append(parts[i]);
            if (i < parts.length - 1) {
                formattedString.append(",<br>");
            }
        }
        formattedString.append("</html>");

        calibrationResultLabel.setText(formattedString.toString());
    }

    public void copyPoseMouseClicked(java.awt.event.MouseEvent evt) throws IOException, UnsupportedFlavorException {
        /** Invoked when the 'Copy pose to clipboard' button is clicked.
         * Copies the pose from the calibration result to the clipboard.
         * To paste the pose the user should connect a USB keyboard to the robot and press 'Ctrl + V'.
         */

        ScriptCommand calibrateAxisCommand = new ScriptCommand("urcap_external_axis_ethercat_calibrate_linear_axis");

        String calibrationResult = calibrationResultLabel.getText();
        if (calibrationResult.equals("No data yet")) {
            JOptionPane.showMessageDialog(null, "No data to copy", "Error", JOptionPane.ERROR_MESSAGE);
            return;
        }
        int poseStart = calibrationResult.indexOf("p[");
        int poseEnd = calibrationResult.indexOf("],", poseStart);
        StringSelection stringSelection = new StringSelection(calibrationResult.substring(poseStart, poseEnd + 1));
        sender.executeURScript(calibrateAxisCommand);

        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        clipboard.setContents(stringSelection, null);
        JOptionPane.showMessageDialog(null, "The calibration pose has been copied to the clipboard.", "Calibration Pose Copied", JOptionPane.INFORMATION_MESSAGE);
    }

    private void calibrateAxisButtonClicked(java.awt.event.MouseEvent evt) {
        /**
         * Invoked when the 'Calibrate Axis' button is clicked.
         * This method initiates a call to the built-in MotionPlus URScript function 'calibrate_linear_axis', using
         * the sampled TCP poses as input.
         * Any potential origin offset of the axis is subtracted from the calibrated pose.
         * The resulting calibration data is parsed and displayed under the 'Calibration Result'.
         */

        try {
            sender.connect("127.0.0.1", ScriptSender.PRIMARY_PORT, 200);

            ScriptCommand calibrateAxisCommand = new ScriptCommand("urcap_external_axis_ethercat_calibrate_linear_axis");

            String poseArray = view.getSamplePosePanel().getPoseArray();
            int originStartIndex = poseArray.indexOf("p[");
            int originEndIndex = poseArray.indexOf("],", originStartIndex) + 1;
            String origin = poseArray.substring(originStartIndex, originEndIndex);
            int xStartIndex = originEndIndex + 2;
            int xEndIndex = poseArray.indexOf("],", xStartIndex) + 1;
            String x_direction = poseArray.substring(xStartIndex, xEndIndex);
            String z_poses = poseArray.substring(xEndIndex + 1);
            isMounted = view.getSamplePosePanel().isRobotMounted();

            calibrateAxisCommand.appendLine(String.format(
                    "calibration = calibrate_linear_axis(%s, [%s], %s, %s)",
                    origin,
                    z_poses,
                    x_direction,
                    isMounted
            ));
            // Subtract any potential origin offset by getting the position of the axis.
            float originOffset = Float.parseFloat(view.getSamplePosePanel().getOriginOffset());
            calibrateAxisCommand.appendLine("calibration[0][2]= calibration[0][2] +" + originOffset);

            setUrsriptCommandLabel(String.format(
                    "calibrate_linear_axis(%s, [%s], %s, %s)",
                    origin,
                    z_poses,
                    x_direction,
                    isMounted
            ));

                String resultString = sender.exportStringFromURScript(calibrateAxisCommand, "calibration");
            if (resultString.length() > 1) {
                resultString = resultString.substring(7, resultString.length() - 1);
                setCalibrationResult(resultString);
                noteLabel.setText(
                    "<html><span style='color:green;'>NOTE:</span><br>" +
                    "See the MotionPlus documentation for how to use the calibration result</span><br><br>");
            }
             else {
                throw new IOException();
            }

        } catch (IOException e) {
            System.err.println("Failed to construct the primary program.");
            return;
        }
    }

    public JButton getCalibrateAxisButton() {
        return calibrateAxisButton;
    }

    public JLabel getCalibrationResultLabel() {
        return calibrationResultLabel;
    }

    public JLabel getCalibrationCommandLabel() {
        return calibrationCommandLabel;
    }

    public JLabel getNoteLabel() {
        return noteLabel;
    }
}
