package com.ur.urcap.externalaxis.linearaxiscalibration.installationNode.installationNodeView;

import com.ur.urcap.externalaxis.linearaxiscalibration.URCommunicator.ScriptCommand;
import com.ur.urcap.externalaxis.linearaxiscalibration.URCommunicator.ScriptSender;
import com.ur.urcap.externalaxis.linearaxiscalibration.installationNode.installationNodeContribution.interfaces.SamplePoseButtonListener;

import javax.swing.*;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import java.awt.*;
import java.awt.event.ItemEvent;

public class SamplePosePanel extends JPanel {
    private final LinearAxisCalibrationInstallationNodeView view;
    private SamplePoseButtonListener samplePoseButtonListener;
    private JTextPane instructionsTextPane;
    private JTextPane sampledPosesTextPane;
    private JButton samplePoseButton;
    private JProgressBar progressBar;
    private JScrollPane sampledPosesScrollPane;
    private String existingText = "";
    private Configuration currentConfiguration;
    private final ScriptSender sender;
    private final JLabel axisNameComboBoxLabel = new JLabel("Axis name:");
    private final JComboBox axisNameComboBox = new JComboBox();

    private final JLabel imageLabel = new JLabel("");
    private final JCheckBox robotOnRailCheckBox = new JCheckBox("Robot mounted on axis");


    private int clickCount = 0;
    static String poseArray = "";

    private static final int progressBarMax = 4;
    private String originOffset;

    public SamplePosePanel(LinearAxisCalibrationInstallationNodeView view) {
        this.view = view;
        sender = new ScriptSender();
        this.currentConfiguration = Configuration.ROBOT_MOUNTED;
        initComponents();
    }

    private void initComponents() {
        int width = 540;
        int height = 626;

        instructionsTextPane = new JTextPane();
        sampledPosesTextPane = new JTextPane();
        sampledPosesScrollPane = new JScrollPane();
        samplePoseButton = new JButton();
        progressBar = new JProgressBar();

        instructionsTextPane.setContentType("text/html");
        instructionsTextPane.setText("<html><body>" +
                "<div style='font-size: 10px;'>" +
                "<p><b>Pre-requisites:</b></p><ul>" +
                "<li>The linear axis must be 'homed' before calibration can be performed.</li>" +
                "<li>The robot should have a pointer tool attached, which has already been calibrated.</li>" +
                "</ul><p style='margin-top: 0;'><b>Calibration Steps:</b></p><ol>" +
                "<li>Hold down the freedrive button and move the robot TCP to the <b>origin</b> of the axis as shown in the figure above</li>" +
                "<li>Release the freedrive button and press the 'Sample Pose' button</li>" +
                "<li>Now sample a pose perpendicular to the linear axis in line with the origin, as indicated by the 'x' in the figure.</li>" +
                "<li>Jog the linear axis to two different positions, e.g. positions '2' and '3' in the figure, and again sample the TCP of the origin at each linear axis position.</li>" +
                "</ol></div></body></html>");

        instructionsTextPane.setFocusable(false);
        instructionsTextPane.setMaximumSize(new Dimension(width-20, 280));

        sampledPosesScrollPane.setViewportView(sampledPosesTextPane);
        sampledPosesScrollPane.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        sampledPosesScrollPane.setPreferredSize(new Dimension(width-20, 130));
        sampledPosesScrollPane.setMaximumSize(new Dimension(width-20, 130));
        sampledPosesScrollPane.setMinimumSize(new Dimension(width-20, 130));
        sampledPosesTextPane.setFocusable(true);

        samplePoseButton.setText("Sample Pose");
        samplePoseButton.setEnabled(false);
        samplePoseButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                samplePoseButtonMouseClicked(evt);
            }
        });

        progressBar.setMaximum(4);
        progressBar.setToolTipText("");
        UIManager.put("blue", new Color(38, 139, 210));
        progressBar.setString(0 + "/"+progressBarMax);
        progressBar.setRequestFocusEnabled(true);
        progressBar.setStringPainted(true);


        JPanel jPanel1 = new JPanel();
        BoxLayout box = new BoxLayout(jPanel1, BoxLayout.Y_AXIS);

        jPanel1.setPreferredSize(new Dimension(width, height));
        jPanel1.setMaximumSize(new Dimension(width, height));
        jPanel1.setMinimumSize(new Dimension(width, height));
        jPanel1.setLayout(box);


        ImageIcon icon = new ImageIcon(getClass().getResource("/LinearCal-Mount.png"));
        imageLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
        imageLabel.setIcon(icon);

        robotOnRailCheckBox.setSelected(true);
        robotOnRailCheckBox.setAlignmentX(Component.LEFT_ALIGNMENT);
        robotOnRailCheckBox.addItemListener(e -> {
            if (e.getStateChange() == ItemEvent.SELECTED) {
                updateContent(Configuration.ROBOT_MOUNTED);
            } else {
                updateContent(Configuration.ROBOT_INDEPENDENT);
            }
            });

        JPanel checkboxPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        checkboxPanel.add(axisNameComboBoxLabel);
        axisNameComboBox.setMinimumSize(new Dimension(150, robotOnRailCheckBox.getPreferredSize().height));
        axisNameComboBox.setMaximumSize(new Dimension(150, robotOnRailCheckBox.getPreferredSize().height));
        axisNameComboBox.setPreferredSize(new Dimension(150, robotOnRailCheckBox.getPreferredSize().height));
        axisNameComboBox.addActionListener(e -> {
            if (axisNameComboBox.getSelectedItem() != null) {
                samplePoseButton.setEnabled(true);
            } else {
                samplePoseButton.setEnabled(false);
            }
        });
        axisNameComboBox.addPopupMenuListener(new PopupMenuListener() {
            @Override
            public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
                axisNameComboBox.removeAllItems();
                String[] names = getAxisNames();
                for (String name : names) {
                    axisNameComboBox.addItem(name);
                }
            }

            @Override
            public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
            }

            @Override
            public void popupMenuCanceled(PopupMenuEvent e) {
            }
        });

        checkboxPanel.add(axisNameComboBox);
        checkboxPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        checkboxPanel.add(robotOnRailCheckBox);
        checkboxPanel.setMaximumSize(new Dimension(width, robotOnRailCheckBox.getPreferredSize().height+10));
        checkboxPanel.setMinimumSize(new Dimension(width, robotOnRailCheckBox.getPreferredSize().height+10));
        checkboxPanel.setPreferredSize(new Dimension(width, robotOnRailCheckBox.getPreferredSize().height+10));

        JButton clearButton = new JButton("Clear");
        clearButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                poseArray = "";
                existingText = "";
                sampledPosesTextPane.setText("");
                progressBar.setValue(0);
                progressBar.setString(0 + "/" + progressBarMax);
                clickCount = 0;
                view.getCalibrationResultPanel().getCalibrateAxisButton().setEnabled(false);
                view.getCalibrationResultPanel().getCalibrationResultLabel().setText("No data yet");
                view.getCalibrationResultPanel().getNoteLabel().setText("");
                view.getCalibrationResultPanel().getCalibrationCommandLabel().setText("No data yet");
            }
        });

        Box horizontalBox = Box.createHorizontalBox();
        horizontalBox.add(Box.createRigidArea(new Dimension(20, 0)));
        horizontalBox.add(samplePoseButton);
        horizontalBox.add(Box.createRigidArea(new Dimension(20, 0)));
        horizontalBox.add(progressBar);
        horizontalBox.add(Box.createRigidArea(new Dimension(20, 0)));
        horizontalBox.add(clearButton);
        horizontalBox.add(Box.createRigidArea(new Dimension(20, 0)));

        add(imageLabel);
        jPanel1.add(checkboxPanel);
        jPanel1.add(instructionsTextPane);
        jPanel1.add(sampledPosesScrollPane);
        jPanel1.add(Box.createRigidArea(new Dimension(0, 10)));
        jPanel1.add(horizontalBox);

        add(jPanel1);
    }

    public void setButtonClickListener(SamplePoseButtonListener listener) {
        this.samplePoseButtonListener = listener;
    }

    private void samplePoseButtonMouseClicked(java.awt.event.MouseEvent evt) {
        /**
        * This method is called when the 'Sample Pose' button is clicked.
        * It increments the progress bar, updates the progress bar text, and calls the onSamplePoseButtonClicked method
        * in the SamplePoseButtonListener interface.
         */
        if (!samplePoseButton.isEnabled()) {
            return;
        }
        clickCount++;
        progressBar.setValue(clickCount);
        progressBar.setString(clickCount + "/" + progressBarMax);
        if (clickCount == 1 && robotOnRailCheckBox.isSelected()) {
            String selectedAxis = (String) axisNameComboBox.getSelectedItem();
            ScriptCommand getOriginOffsetCommand = new ScriptCommand("get_origin_offset");
            getOriginOffsetCommand.appendLine("origin_offset = get_pose(\"" + selectedAxis + "\", \"world\", \"world\")[2]");
            originOffset = sender.exportStringFromURScript(getOriginOffsetCommand, "origin_offset");
        }
        if (clickCount >= progressBarMax) {
            view.getCalibrationResultPanel().getCalibrateAxisButton().setEnabled(true);
        }
        samplePoseButtonListener.onSamplePoseButtonClicked();
    }

    public void setLabelText(String text) {
        /**
        * This method is called when a pose is sampled. It updates the text in the sampledPosesTextPane.
        * The text is formatted to show only 5 decimal places.
        * The text is also appended to the poseArray string, which is used to send the poses to the robot.
        */
        text = text.replaceAll("(\\d+\\.\\d{5})\\d+", "$1");

        sampledPosesTextPane.setContentType("text/html");
        if (!existingText.isEmpty()) {
            poseArray += ", ";
            poseArray += "p"+text;
            existingText = existingText.concat("<br>p" + text + "<br>");
        }
        else {
            poseArray += "p"+text;
            existingText = existingText.concat("p" + text + "<br>");
        }
        sampledPosesTextPane.setText(existingText);
    }

    public String getPoseArray() {
        return poseArray;
    }

    public void updateContent(Configuration configuration) {
        /**
         * This method updates the content of the panel based on the configuration of the robot.
         * If the robot is mounted on the linear axis, the image is updated to show the robot mounted on the axis.
         */
        this.currentConfiguration = configuration;
        if (configuration == Configuration.ROBOT_MOUNTED) {
            ImageIcon icon = new ImageIcon(getClass().getResource("/LinearCal-Mount.png"));
            imageLabel.setIcon(icon);

        } else if (configuration == Configuration.ROBOT_INDEPENDENT) {
            ImageIcon icon = new ImageIcon(getClass().getResource("/LinearCal-NoMount.png"));
            imageLabel.setIcon(icon);
        }
    }

    public enum Configuration {
        ROBOT_MOUNTED,
        ROBOT_INDEPENDENT
    }

    public Configuration getCurrentConfiguration() {
        return currentConfiguration;
    }

    public String isRobotMounted() {
        return robotOnRailCheckBox.isSelected() ? "True" : "False";
    }


    public
    String getAxisNamesScript() {
        /**
         * This method generates the script to get the names of the axes in the robot model.
         * The script uses a recursive function to traverse the robot model and get the names of the axes.
         */
        StringBuilder programBuilder = new StringBuilder();

        programBuilder.append("  def get_axes_recursively(node, stringAxes):\n");
        programBuilder.append("    axes = get_children(node)\n");

        programBuilder.append("    index = 0\n");
        programBuilder.append("    while index < length(axes):\n");
        programBuilder.append("      axis = axes[index]\n");
        programBuilder.append("      if (node == \"world\"):\n");
        programBuilder.append("        stringAxes = get_axes_recursively(axis, stringAxes)\n");
        programBuilder.append("      elif axis != \"flange\" and axis != \"world\" and axis != \"base\":\n");
        programBuilder.append("        stringAxes = stringAxes + axis\n");
        programBuilder.append("        stringAxes = stringAxes + \",\"\n");
        programBuilder.append("        stringAxes = get_axes_recursively(axis, stringAxes)\n");
        programBuilder.append("      end\n");
        programBuilder.append("      index = index + 1\n");
        programBuilder.append("    end\n");
        programBuilder.append("    return stringAxes\n");
        programBuilder.append("  end\n");

        programBuilder.append("  str = get_axes_recursively(\"world\", \"\")\n");

        return programBuilder.toString();
    }

    public
    String[] getAxisNames() {
        /**
         * This method connects a sender object and exports a string from the ur script.
         * The string is then split and returned as an array of axis names.
         */
        sender.connect("127.0.0.1", ScriptSender.PRIMARY_PORT, 200);


        ScriptCommand command = new ScriptCommand("get_axes");
        command.appendLine(getAxisNamesScript());


        final String worldModelString =
                sender.exportStringFromURScript(command, "str");
        return worldModelString.split(",");
    }

    public String getOriginOffset() {
        return originOffset;
    }
}




