# Move-To URCap Example
## Description
The Move-To URCap provides a working example for moving the given axis group via controls presented in the toolbar.
## Build
* The URCap SDK must be installed and available in the build environment
1.
   ```bash
   cd move-to
   ```
1.
   ```bash
   ./build.sh
   ```
   Note: By default the build script will attempt to build the URCap then deploy to a local simulator (if available).
## Example Usage
* Note: The Move-To URCap does not manage the lifecycle of any world-model or axis-group entities.
1. On the robot, execute a program or script that creates 2 Axis Groups and 3 Axes
   * The program/script should finish without removing the Axis Group. Remember to configure the axis as well, the EtherCAT Axis Config URCap could also be used for this purpose.
   * Example script:
     ```ruby
     AXIS_TYPE = 0 # 0:rotary, 1:linear
     AXIS_VELOCITY_LIMIT = d2r(360) # rad/s
     AXIS_ACCELERATION_LIMIT = d2r(3600) # rad/s^2
     AXIS_ENCODER_RESOLUTION = 3600 # counts/rev
     AXIS_FEED_CONSTANT = d2r(360) # rad/rev
     AXIS_GEAR_RATIO = 1.0
     AXIS_ZERO_OFFSET = 0

     reset_world_model()
     axis_group_add("group1", p[0, 0, 0, 0, 0, 0], "world")
     axis_group_add_axis("group1", "axis1", "", p[0, 0, 0, 0, 0, 0], AXIS_TYPE, AXIS_VELOCITY_LIMIT, AXIS_ACCELERATION_LIMIT)
     axis_group_add_axis("group1", "axis2", "", p[0, 0, 0, 0, 0, 0], AXIS_TYPE, AXIS_VELOCITY_LIMIT, AXIS_ACCELERATION_LIMIT)
     axis_group_add("group2", p[0, 0, 0, 0, 0, 0], "world")
     axis_group_add_axis("group2", "axis3", "", p[0, 0, 0, 0, 0, 0], AXIS_TYPE, AXIS_VELOCITY_LIMIT, AXIS_ACCELERATION_LIMIT)
     ```
1. After the program has finished executing, open the Move To panel of the toolbar
   ![open toolbar](./docs/images/open_toolbar.png)
1. Select the group to move from the Axis Group dropdown
   ![select group](./docs/images/select_group.png)
1. Enter the target positions specific to the desired Axis Group to move
   ![toolbar parameters](./docs/images/enter_target.png)
1. Move specified Axis Group by holding down the 'Move to Target Positions' button
   * The background color will change from white to grey indicating that the motion is in progress. The `Sent Command` field will update to indicate the command that has been sent to the robot via the Primary Interface
   * If the 'Frame Tracking' checkbox is checked, the robot will perform frame tracking as the axis group is moving.
   ![move group](./docs/images/move_group.png)
1. Halt jogging by releasing the held-down button
   * The axis group motion will halt, and the background color will change back to white indicating that the motion has stopped. 
