package com.ur.urcap.externalaxis.rotaryaxiscalibration.installationNode.installationNodeView;

import com.ur.urcap.externalaxis.rotaryaxiscalibration.URCommunicator.ScriptCommand;
import com.ur.urcap.externalaxis.rotaryaxiscalibration.URCommunicator.ScriptSender;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.io.IOException;

public class CalibrationResultPanel extends JPanel {
    private final JLabel calibrationResultLabel;
    private final JLabel calibrationCommandLabel;

    private final RotaryAxisCalibrationInstallationNodeView view;

    private JButton calibrateAxisButton = new JButton();

    private final ScriptSender sender;

    public CalibrationResultPanel(RotaryAxisCalibrationInstallationNodeView view) {
        this.view = view;
        sender = new ScriptSender();

        calibrateAxisButton.setText("Calibrate Axis");
        calibrateAxisButton.setEnabled(false);
        calibrateAxisButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                calibrateAxisButtonClicked(evt);

            }
        });

        JLabel calibrationCommandLabel = new JLabel("<html><b><u>URScript Command:</u></b></html>");
        JLabel calibrationResultLabel = new JLabel("<html><b><u>Calibration Result:</u></b></html>");

        this.calibrationCommandLabel = new JLabel("No data yet");
        this.calibrationResultLabel = new JLabel("No data yet");

        JButton copyPoseButton = new JButton("Copy pose to clipboard");
        copyPoseButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                copyPoseMouseClicked(evt);
            }
        });

        int width = 480;
        int height = 220;
        JPanel scriptCommandPanel = new JPanel();
        BoxLayout boxLayout = new BoxLayout(scriptCommandPanel, BoxLayout.Y_AXIS);

        scriptCommandPanel.setPreferredSize(new Dimension(width, height));
        scriptCommandPanel.setMaximumSize(new Dimension(width, height));
        scriptCommandPanel.setLayout(boxLayout);

        scriptCommandPanel.add(Box.createRigidArea(new Dimension(0, 20)));
        scriptCommandPanel.add(calibrationCommandLabel);
        scriptCommandPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        scriptCommandPanel.add(this.calibrationCommandLabel);
        add(scriptCommandPanel);

        JPanel resultPanel = new JPanel();
        BoxLayout boxResultPanel = new BoxLayout(resultPanel, BoxLayout.Y_AXIS);

        resultPanel.setPreferredSize(new Dimension(width, 290));
        resultPanel.setMaximumSize(new Dimension(width, 290));
        resultPanel.setLayout(boxResultPanel);

        resultPanel.add(calibrationResultLabel);
        resultPanel.add(Box.createRigidArea(new Dimension(0, 10)));
        resultPanel.add(this.calibrationResultLabel);
        add(resultPanel);

        JPanel boxPanel = new JPanel();
        Box horizontalBox = Box.createHorizontalBox();
        horizontalBox.add(calibrateAxisButton);
        horizontalBox.add(Box.createRigidArea(new Dimension(40, 0)));
        horizontalBox.add(copyPoseButton);
        boxPanel.add(horizontalBox);
        add(boxPanel);
    }

    public void setUrsriptCommandLabel(String calibrationCommand) {
        calibrationCommand = "<html>" + calibrationCommand.replace("p", "<br>p") + "</html>";
        calibrationCommandLabel.setText(calibrationCommand);
    }

    public void setCalibrationResult(String calibrationResult) {
        /** Formats the calibration result string in order to be properly displayed.
         * The string is split after each comma except for the line containing the pose.
         */

        calibrationResult = calibrationResult.replaceAll("(\\d+\\.\\d{4})\\d+", "$1");

        String[] parts = calibrationResult.split(", ");

        StringBuilder formattedString = new StringBuilder("<html>");
        for (int i = 0; i < parts.length; i++) {
            formattedString.append(parts[i]);
            if (i < parts.length - 1) {
                formattedString.append(",<br>");
            }
        }
        formattedString.append("</html>");

        calibrationResultLabel.setText(formattedString.toString());
    }

    public void copyPoseMouseClicked(java.awt.event.MouseEvent evt) {
        /** Invoked when the 'Copy pose to clipboard' button is clicked.
         * Copies the pose from the calibration result to the clipboard.
         */

        String calibrationResult = calibrationResultLabel.getText();
        if (calibrationResult.equals("No data yet")) {
            JOptionPane.showMessageDialog(null, "No data to copy", "Error", JOptionPane.ERROR_MESSAGE);
            return;
        }
        int poseStart = calibrationResult.indexOf("p[");
        int poseEnd = calibrationResult.indexOf("],", poseStart);
        StringSelection stringSelection = new StringSelection(calibrationResult.substring(poseStart, poseEnd + 1));
        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        clipboard.setContents(stringSelection, null);
        JOptionPane.showMessageDialog(null, "Pose copied to clipboard");
    }

    private void calibrateAxisButtonClicked(java.awt.event.MouseEvent evt) {
        /**
         * Invoked when the 'Calibrate Axis' button is clicked.
         * This method initiates a call to the built-in MotionPlus URScript function 'calibrate_rotary_axis', using
         * the sampled TCP poses as input.
         * The resulting calibration data is parsed and displayed in the 'Generated Script' tab.
         */

        try {
            sender.connect("127.0.0.1", ScriptSender.PRIMARY_PORT, 200);

            ScriptCommand calibrateAxisCommand = new ScriptCommand("urcap_external_axis_ethercat_calibrate_axis");

            String poseArray = view.getSamplePosePanel().getPoseArray();
            calibrateAxisCommand.appendLine(String.format(
                    "calibration = calibrate_rotary_axis([%s])",
                    poseArray
            ));

            poseArray = poseArray.replaceAll("(\\d+\\.\\d{4})\\d+", "$1");

            if (sender.executeURScript(calibrateAxisCommand)) {
                setUrsriptCommandLabel(String.format("calibrate_rotary_axis([%s])", poseArray));

                String resultString = sender.exportStringFromURScript(calibrateAxisCommand, "calibration");
                if (resultString.length() > 1) {
                    resultString = resultString.substring(7, resultString.length() - 1);
                    setCalibrationResult(resultString);
                }


            } else {
                throw new IOException();
            }

        } catch (IOException e) {
            System.err.println("Failed to construct the primary program.");
            return;
        }
    }

    public JButton getCalibrateAxisButton() {
        return calibrateAxisButton;
    }

    public JLabel getCalibrationResultLabel() {
        return calibrationResultLabel;
    }

    public JLabel getCalibrationCommandLabel() {
        return calibrationCommandLabel;
    }
}
