package com.ur.urcap.externalaxis.rotaryaxiscalibration.installationNode.installationNodeView;

import com.ur.urcap.externalaxis.rotaryaxiscalibration.installationNode.installationNodeContribution.interfaces.SamplePoseButtonListener;

import javax.swing.*;
import java.awt.*;

public class SamplePosePanel extends JPanel {
    private final RotaryAxisCalibrationInstallationNodeView view;
    private SamplePoseButtonListener samplePoseButtonListener;
    private JTextPane instructionsTextPane;
    private JTextPane sampledPosesTextPane;
    private JButton samplePoseButton;
    private JProgressBar progressBar;
    private JScrollPane sampledPosesScrollPane;
    private String existingText = "";

    private int clickCount = 0;
    static String poseArray = "";

    private static final int progressBarMax = 4;

    public SamplePosePanel(RotaryAxisCalibrationInstallationNodeView view) {
        this.view = view;
        initComponents();
    }

    private void initComponents() {
        int width = 480;
        int height = 626;

        instructionsTextPane = new JTextPane();
        sampledPosesTextPane = new JTextPane();
        sampledPosesScrollPane = new JScrollPane();
        samplePoseButton = new JButton();
        progressBar = new JProgressBar();

        instructionsTextPane.setContentType("text/html");
        instructionsTextPane.setText("<html><body>" +
                "<div style='font-size: 10px;'>" +
                "<p><b>Pre-requisites:</b></p><ul>" +
                "<li>The rotary axis must be 'homed' before calibration can be performed.</li>" +
                "<li>The robot should have a pointer tool attached, which has already been calibrated.</li>" +
                "<li>The MoveTo URCap must be installed.</li>" +
                "</ul><p style='margin-top: 0;'><b>Calibration Steps:</b></p>" +
                "Calibration involves sampling the same point 4 times around the rotation axis. " +
                "In between samples, the rotary axis should rotate in the positive direction." +
                "<ol>" +
                "<li>Hold down the freedrive button on the Teach Pendant.</li>" +
                "<li>Move the robot to the desired point and release the freedrive button.</li>" +
                "<li>Press the Sample Pose button below.</li>" +
                "<li>Go to the MoveTo URCap and increment the angle by 90 degrees.</li>" +
                "<li>Repeat this process until the same point has been sampled 4 times.</li>" +
                "<li>Press the Calibrate Axis button.</li>" +
                "</ol>" +
                "</body></html>");
        instructionsTextPane.setFocusable(false);
        instructionsTextPane.setMaximumSize(new Dimension(width, 375));

        sampledPosesScrollPane.setViewportView(sampledPosesTextPane);
        sampledPosesScrollPane.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        sampledPosesScrollPane.setPreferredSize(new Dimension(width, 130));
        sampledPosesScrollPane.setMaximumSize(new Dimension(width, 130));
        sampledPosesScrollPane.setMinimumSize(new Dimension(width, 130));
        sampledPosesTextPane.setFocusable(true);

        samplePoseButton.setText("Sample Pose");
        samplePoseButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                samplePoseButtonTab2MouseClicked(evt);
            }
        });

        progressBar.setMaximum(4);
        progressBar.setToolTipText("");
        UIManager.put("blue", new Color(38, 139, 210));
        progressBar.setString(0 + "/"+progressBarMax);
        progressBar.setRequestFocusEnabled(true);
        progressBar.setStringPainted(true);

        JPanel jPanel1 = new JPanel();
        BoxLayout box = new BoxLayout(jPanel1, BoxLayout.Y_AXIS);

        jPanel1.setPreferredSize(new Dimension(width, height));
        jPanel1.setMaximumSize(new Dimension(width, height));
        jPanel1.setMinimumSize(new Dimension(width, height));
        jPanel1.setLayout(box);

        jPanel1.add(instructionsTextPane);

        jPanel1.add(sampledPosesScrollPane);
        jPanel1.add(Box.createRigidArea(new Dimension(0, 20)));

        JButton clearButton = new JButton("Clear");
        clearButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                poseArray = "";
                existingText = "";
                sampledPosesTextPane.setText("");
                progressBar.setValue(0);
                progressBar.setString(0 + "/" + progressBarMax);
                clickCount = 0;
                view.getCalibrationResultPanel().getCalibrateAxisButton().setEnabled(false);
                view.getCalibrationResultPanel().getCalibrationResultLabel().setText("No data yet");
                view.getCalibrationResultPanel().getCalibrationCommandLabel().setText("No data yet");
            }
        });

        Box horizontalBox = Box.createHorizontalBox();
        horizontalBox.add(Box.createRigidArea(new Dimension(20, 0)));
        horizontalBox.add(samplePoseButton);
        horizontalBox.add(Box.createRigidArea(new Dimension(20, 0)));
        horizontalBox.add(progressBar);
        horizontalBox.add(Box.createRigidArea(new Dimension(20, 0)));
        horizontalBox.add(clearButton);
        horizontalBox.add(Box.createRigidArea(new Dimension(20, 0)));

        jPanel1.add(horizontalBox);

        add(jPanel1);
    }

    public void setButtonClickListener(SamplePoseButtonListener listener) {
        this.samplePoseButtonListener = listener;
    }

    private void samplePoseButtonTab2MouseClicked(java.awt.event.MouseEvent evt) {
        clickCount++;
        progressBar.setValue(clickCount);
        progressBar.setString(clickCount + "/" + progressBarMax);
        if (clickCount >= progressBarMax) {
            view.getCalibrationResultPanel().getCalibrateAxisButton().setEnabled(true);
        }
        samplePoseButtonListener.onSamplePoseButtonClicked();
    }

    public void setLabelText(String text) {
        text = text.replaceAll("(\\d+\\.\\d{5})\\d+", "$1");

        sampledPosesTextPane.setContentType("text/html");
        if (!existingText.isEmpty()) {
            poseArray += ", ";
            poseArray += "p"+text;
            existingText = existingText.concat("<br>p" + text + "<br>");
        }
        else {
            poseArray += "p"+text;
            existingText = existingText.concat("p" + text + "<br>");
        }
        sampledPosesTextPane.setText(existingText);
    }

    public String getPoseArray() {
        return poseArray;
    }
}




